#!/usr/local/bin/ruby -Ku
#--*-coding:utf-8-*--
$LOAD_PATH << File.dirname(__FILE__) + "/src_ruby"

#CGI通信の主幹クラス用ファイル
#ファイルアップロード系以外は全てこのファイルへ通知が送られます。
#クライアント通知されたJsonデータからセーブデータ(.jsonテキスト)を読み出し・書き出しするのが主な作業。
#変更可能な設定は config.rb にまとめているため、環境設定のためにこのファイルを変更する必要は基本的には無いです。

require 'jcode'
require 'kconv'
require 'cgi'
require 'stringio'
require 'logger'
require 'uri'
require 'fileutils'
require 'tempfile'
require 'json/jsonParser'

require "config.rb"
require "loggingFunction.rb"
require "FileLock.rb"
require "saveDirInfo.rb"
require "card.rb"


$SAFE = 1

$card = Card.new();



#サーバCGIとクライアントFlashのバージョン一致確認用
$version = "Ver.1.22.00(2009/10/17)"

$saveFileNames = File.join($saveDataTempDir, 'saveFileNames.json');
$imageUrlText = File.join($imageUploadDir, 'imageUrl.txt');

$chatMessageDataLogAll = 'chatLongLines.txt'

$loginUserInfo = 'login.json'
$playRoomInfo = 'playRoomInfo.json'

$saveFiles = {
  'chatMessageDataLog' => 'chat.json',
  'map' => 'map.json',
  'characters' => 'characters.json',
  'time' => 'time.json',
  'effects' => 'effects.json',
};



class DodontoFServer
  
  def getRequestData(key)
    logging(key, "getRequestData key")
    
    content_type = @content_type
    logging(content_type, "@cgi.content_type")
    
    #if( content_type == 'application/octet-stream')
    if( %r|^multipart/form-data| === content_type )
      logging("content_type is fileUploader")
      return getRequestDataForFileUploader(key)
    end
    
    logging("content_type is NOT fileUploader")
    value = @cgi[key]
    
    return value
  end

  def getRequestDataJsonData(key)
    jsonData = getRequestData(key)
    return getJsonDataFromText(jsonData)
  end
  
  def getStringFromStringOrStringIoOrFile(input)
    logging(input, 'input')
    logging(input.class, 'input.class')
    
    if( input.instance_of?( StringIO ) )
      return input.string
    end
    
    if( input.instance_of?( File ) or input.instance_of?( Tempfile ) )
      output = input.readlines().join
      logging(output, "getStringFromStringOrStringIoOrFile output")
      result = URI.unescape(output)
      logging(result, "getStringFromStringOrStringIoOrFile result")
      return result
    end
    
    return input
  end
  
  def getRequestDataForFileUploader(key)
    logging(key, "getRequestDataForFileUploader key")
    
    @cgiParams ||= @cgi.params
    logging(@cgiParams, "SaveFileUploader cgi.params")
    
    logging('pass1')
    
    if( @cgiParams.include?(key) )
      value = @cgi[key]
      return value if( key == 'Filedata' )
      
      return getStringFromStringOrStringIoOrFile(value)
    end
    
    logging('pass2')
    
    @jsonDataForFileUploader ||= getJsonDataForFileUploader
    logging(@jsonDataForFileUploader, "@jsonDataForFileUploader")
    
    logging('pass4')
    
    return @jsonDataForFileUploader[key]
  end
  
  def getJsonDataForFileUploader
    logging("get @cgi['__jsonDataForFileUploader__']");
    jsonDataIo = @cgi['__jsonDataForFileUploader__']
    logging(jsonDataIo, "jsonDataIo");
    
    jsonDataString = getStringFromStringOrStringIoOrFile(jsonDataIo)
    logging(jsonDataString, "jsonDataString in getRequestData")
    
    logging('pass3')
    
    logging("getJsonDataFromText(jsonDataString)")
    jsonDataForFileUploader = ""
    begin
      isRiseError = true
      jsonDataForFileUploader = getJsonDataFromText(jsonDataString, isRiseError)
    rescue => e
      jsonDataString = CGI.unescape(jsonDataString)
      jsonDataForFileUploader = getJsonDataFromText(jsonDataString)
    end
    logging(jsonDataForFileUploader, "jsonDataForFileUploader");
    
    return jsonDataForFileUploader
  end
  
  def initialize(cgi, content_type)
    @cgi = cgi
    @content_type = content_type
    
    @saveDirInfo = SaveDirInfo.new(getRequestData("saveDataDirIndex"), $saveDataMaxCount, $SAVE_DATA_DIR)
    
    @isSampleMode = false
    if( $isSampleModeEnable )
      isSampleMode = getRequestData("isSampleMode")
      @isSampleMode = (isSampleMode == "true")
      logging(@isSampleMode, "isSampleMode")
      
      if( @isSampleMode )
        @saveDirInfo.setSampleMode
      end
    end
    
    
    @saveFiles = {}
    $saveFiles.each do |saveDataKeyName, saveFileName|
      logging(saveDataKeyName, "saveDataKeyName")
      logging(saveFileName, "saveFileName")
      @saveFiles[saveDataKeyName] = @saveDirInfo.getTrueSaveFileName(saveFileName)
    end
  end
  
  def getSaveFileLock(saveFileName)
    begin
      return FileLock.new(saveFileName + ".lock")
    rescue => e
      loggingForce(@saveDirInfo.inspect, "when getSaveFileLock error : @saveDirInfo.inspect");
      raise
    end
  end
  
  def loadSaveFile(typeName, saveFileName, lastUpdateTimes)
    saveData = nil
    
    begin
      if( $IS_SAVE_LONG_CHAT_LOG and typeName == 'chatMessageDataLog' and lastUpdateTimes[typeName] == 0 )
        saveFileName = @saveDirInfo.getTrueSaveFileName($chatMessageDataLogAll)
        saveFileLock = getSaveFileLock(saveFileName)
        
        saveFileLock.lock do
          chatMessageDataLog = []
          
          lines = []
          if( File.exist?(saveFileName) )
            lines = File.readlines(saveFileName)
          end
          
          lines.each do |line|
            chatMessageDataLog << getJsonDataFromText(line.chomp)
          end
          
          saveData = {"chatMessageDataLog" => chatMessageDataLog}
          lastUpdateTimes[typeName] = getSaveFileTimeStamp(saveFileName);
        end
      else
        saveFileLock = getSaveFileLock(saveFileName)
        
        saveFileLock.lock do
          saveData = getSaveData(saveFileName)
          lastUpdateTimes[typeName] = getSaveFileTimeStamp(saveFileName);
        end
      end 
    rescue => e
      loggingForce($!.inspect )
      loggingForce(e.inspect )
      # raise e
    end
    
    logging(saveData.inspect, saveFileName)
    
    return saveData
  end
  
  
  def changeSaveData(saveFileName)
    #loggingForce(saveFileName, "changeSaveData saveFileName")
    saveFileLock = getSaveFileLock(saveFileName)
    
    saveFileLock.lock do
      saveData = getSaveData(saveFileName)
      yield(saveData)
      
      #logging(saveData.inspect, saveFileName)
      
      text = getTextFromJsonData(saveData)
      
      createFile(saveFileName, text)
    end
  end
  
  def createSaveFile(saveFileName, text)
    existFiles = nil
    
    changeSaveData($saveFileNames) do |saveData|
      existFiles = saveData["fileNames"]
      existFiles ||= []
      
      if( existFiles.include?(saveFileName) ) 
        existFiles << saveFileName
      end
      
      createFile(saveFileName, text)
    end
    
    logging(existFiles, 'createSaveFile existFiles')
  end
  
  def createFile(saveFileName, text)
    begin
      File.open(saveFileName, "w+") do |file|
        file.write(text.toutf8)
      end
    rescue => e
      loggingForce($!.inspect )
      loggingForce(e.inspect )
      raise e
    end
  end
  
  def getTextFromJsonData(jsonData)
    return JsonBuilder.new.build(jsonData)
  end
  
  def escapeHtmlDataFromJson(jsonData)
    if( jsonData.kind_of?(String) )
      escapedString = CGI.escapeHTML(jsonData)
      jsonData.gsub!(/\A.*\z/, escapedString);
    elsif( jsonData.kind_of?(Hash) )
      jsonData.each do |key, value|
        escapeHtmlDataFromJson(value)
      end
    elsif( jsonData.kind_of?(Array) )
      jsonData.each do |item|
        escapeHtmlDataFromJson(item)
      end
    else
      # do nothing
    end
    
  end
  
  def getJsonDataFromText(text, isRiseError = false)
    jsonData = nil
    begin
      logging(text, "getJsonDataFromText start")
      jsonData = JsonParser.new.parse(text)
      logging("getJsonDataFromText end")
    rescue => e
      loggingForce($!.inspect )
      loggingForce(e.inspect )
      if( isRiseError )
        raise e
      end
      jsonData = {}
    end
    
    return jsonData
  end
  
  def getSaveData(fileName)
    return {} unless( File.exist?(fileName) )
    
    text = ''
    open(fileName, 'r') do |file|
      text = file.read
    end
    
    saveData = getJsonDataFromText(text)
    
    return saveData
  end
  
  def analyzeCommand
    commandName = getRequestData('Command')
    
    #    loggingForce(commandName, "commandName")
    
    if( commandName.nil? or commandName.empty? )
      return getTestResponseText
    end
    
    hasReturn = "hasReturn";
    hasNoReturn = "hasNoReturn";
    
    commands = [
      ['refresh', hasReturn],
      
      ['getImageList', hasReturn], 
      ['getGraveyardCharacterData', hasReturn], 
      ['getLoginInfo', hasReturn], 
      ['deleteImage', hasReturn], 
      ['uploadImageUrl', hasReturn], 
      ['saveChatLog', hasReturn], 
      ['save', hasReturn], 
      ['load', hasReturn], 
      ['uploadReplayData', hasReturn], 
      ['checkRoomStatus', hasReturn], 
      ['login', hasReturn], 
      ['uploadImageFile', hasReturn], 
      ['createPlayRoom', hasReturn], 
      ['removePlayRoom', hasReturn], 
      ['getImageTags', hasReturn], 
      ['addCharacter', hasReturn],
      
      ['logout', hasNoReturn], 
      ['addCard', hasNoReturn],
      ['changeCharacterData', hasNoReturn],
      ['changeCounterNames', hasNoReturn],
      ['removeCharacter', hasNoReturn],
      ['initCards', hasNoReturn],
      ['returnCard', hasNoReturn],
      ['shuffleCards', hasNoReturn],
      ['dumpTrushCards', hasNoReturn],
      ['clearCharacterByType', hasNoReturn],
      ['drawCard', hasNoReturn],
      ['resurrectCharacter', hasNoReturn],
      ['moveCharacter', hasNoReturn],
      ['changeMap', hasNoReturn],
      ['sendChatMessage', hasNoReturn],
      ['sendChatMessageMany', hasNoReturn],
      ['changeRoundTime', hasNoReturn],
      ['addEffect', hasNoReturn], 
      ['changeEffect', hasNoReturn], 
      ['removeEffect', hasNoReturn], 
      ['changeImageTags', hasNoReturn], 
    ]
    
    commands.each do |command, commandType|
      next unless( command == commandName )
      logging(command, "command")
      logging(commandType, "commandType")
      
      case commandType
      when hasReturn
        return eval( command )
      when hasNoReturn
        eval( command )
        return nil
      end
    end
    
    throw Exception.new("\"#{commandName}\" is invalid command")
    
  end
  
  def getTestResponseText
    "「どどんとふ」の動作環境は正常に起動しています。";
  end
  
  
  def getCurrentSaveData(lastUpdateTimes)
    @saveFiles.each do |saveFileTypeName, saveFileName|
      logging(saveFileTypeName, "saveFileTypeName");
      logging(saveFileName, "saveFileName");
      targetLastUpdateTime = lastUpdateTimes[saveFileTypeName];
      logging(targetLastUpdateTime, "targetLastUpdateTime");
      
      if( isSaveFileChanged(targetLastUpdateTime, saveFileName) )
        logging(saveFileName, "saveFile is changed");
        targetSaveData = loadSaveFile(saveFileTypeName, saveFileName, lastUpdateTimes)
        yield(targetSaveData, saveFileTypeName)
      end
    end
  end
  
  def refresh
    logging("==>Begin refresh");
    
    refreshDataJsonString = getRequestData('refreshData')
    logging(refreshDataJsonString, "refreshDataJsonString");
    refreshData = getJsonDataFromText(refreshDataJsonString)
    
    lastUpdateTimes = refreshData['lastUpdateTimes']
    logging(lastUpdateTimes, "lastUpdateTimes");
    
    refreshIndex = refreshData['refreshIndex'];
    logging(refreshIndex, "refreshIndex");
    
    now = Time.now
    whileLimitTime = now + $refreshTimeout
    logging(now, "now")
    logging($refreshTimeout, "refreshTimeout")
    logging(whileLimitTime, "whileLimitTime")
    
    saveData = {}
    while( Time.now < whileLimitTime )
      getCurrentSaveData(lastUpdateTimes) do |targetSaveData, saveFileTypeName|
        saveData.merge!(targetSaveData)
      end
      
      logging(saveData, "saveData is empty?");
      break unless( saveData.empty? )
      logging("saveData is empty!");
      
      logging("sleep...");
      sleep( $refreshInterval )
      logging("awake.");
    end
    
    logging("refresh BREAK");
    
    needSave = true
    uniqueId = refreshData['uniqueId'];
    userName = refreshData['userName'];
    loginUserInfoSaveFile = @saveDirInfo.getTrueSaveFileName($loginUserInfo)
    
    loginUserInfo = updateLoginUserInfo(loginUserInfoSaveFile, userName, uniqueId)
    
    saveData['lastUpdateTimes'] = lastUpdateTimes;
    saveData['refreshIndex'] = refreshIndex;
    saveData['loginUserInfo'] = loginUserInfo;
    saveData['cardMount'] = {}
    saveData['cardTrushMount'] = {}
    logging("==>End refresh");
    
    logging(loginUserInfo, "loginUserInfo");
    
    return saveData
  end
  
  def updateLoginUserInfo(trueSaveFileName, userName = '', uniqueId = '')
    logging(uniqueId, 'updateLoginUserInfo uniqueId')
    logging(userName, 'updateLoginUserInfo userName')
    
    result = []
    
    return result if( uniqueId == -1 )
    
    nowSeconds = Time.now.to_i
    logging(nowSeconds, 'nowSeconds')
    
    changeSaveData(trueSaveFileName) do |saveData|
      changeUserInfo(saveData, uniqueId, nowSeconds, userName)
      
      saveData.delete_if do |existUserId, userInfo|
        isDeleteUserInfo?(existUserId, userInfo, nowSeconds)
      end
      
      saveData.keys.sort.each do |userId|
        userInfo = saveData[userId]
        data = {
          "userName" => userInfo['userName'],
          "userId" => userId, 
        }
        result << data
      end
    end
    
    return result
  end
  
  def isDeleteUserInfo?(existUserId, userInfo, nowSeconds)
    isLogout = userInfo['isLogout']
    return true if(isLogout)
    
    timeSeconds = userInfo['timeSeconds']
    diffSeconds = nowSeconds - timeSeconds
    return ( diffSeconds > $loginTimeOut )
  end
  
  def changeUserInfo(saveData, uniqueId, nowSeconds, userName)
    return if( uniqueId.empty? )
    
    isLogout = false
    if( saveData.include?(uniqueId) )
      isLogout = saveData[uniqueId]['isLogout']
    end
    
    return if( isLogout )
    
    saveData[uniqueId] = {
      'userName'=>userName,
      'timeSeconds'=>nowSeconds,
    }
  end
  
  def addTextsCharacterImageList(imageList, *texts)
    texts.each do |text|
      next unless( File.exist?(text) )
      
      lines = File.readlines(text)
      lines.each do |line|
        line.chomp!
        
        next if(line.empty?)
        next if(imageList.include?(line))
        
        imageList << line
      end
    end
  end
  
  def saveChatLog()
    chatLogHtml = getRequestData('chatLogHtml')
    saveFileName = 'chatLog.html'
    trueSaveFileName = @saveDirInfo.getTrueSaveFileName(saveFileName)
    
    begin
      File.open(trueSaveFileName, "w+") do |file|
        file.write(chatLogHtml.toutf8)
      end
    rescue => e
      loggingForce("saveChatLog error!!!!")
      loggingForce($!.inspect )
      loggingForce(e.inspect )
      raise e
    end
    
    result = {"chatLogFileName" => trueSaveFileName}
    return result
  end
  
  
  def getPlayRoomPasswordLockStates()
    passwordLockStates = {}
    @saveDirInfo.getMaxCount.times{|i| passwordLockStates[i] = false}
    logging(passwordLockStates, 'passwordLockStates')
    
    @saveDirInfo.each_with_index($playRoomInfo) do |saveFiles, index|
      if( saveFiles.size != 1 )
        loggingForce("[#{index}](getPlayRoomPasswordLockStates) invalid playRoomInfo saveFiles:#{saveFiles.inspect}")
        next
      end
      
      trueSaveFileName = saveFiles.first
      
      changeSaveData(trueSaveFileName) do |saveData|
        password = saveData['playRoomChangedPassword']
        unless( password.nil? )
          passwordLockStates[index] = true
        end
      end
    end
    
    logging(passwordLockStates, 'passwordLockStates')
    
    return passwordLockStates
  end
  
  def getPlayRoomNames()
    playRoomNames = {}
    @saveDirInfo.getMaxCount.times{|i| playRoomNames[i] = "（空き部屋）"}
    logging(playRoomNames, 'playRoomNames')
    
    @saveDirInfo.each_with_index($saveFiles['map']) do |saveFiles, index|
      
      if( saveFiles.size != 1 )
        loggingForce("[#{index}](getPlayRoomNames) invalid playRoomInfo saveFiles:#{saveFiles.inspect}")
        next
      end
      
      @saveDirInfo.setSaveDataDirIndex(index)
      trueSaveFileName = @saveDirInfo.getTrueSaveFileName($playRoomInfo)
      
      changeSaveData(trueSaveFileName) do |saveData|
        playRoomName = saveData['playRoomName']
        playRoomName ||= "プレイルームNo.#{index}"
        playRoomNames[index] = playRoomName
      end
    end
    logging(playRoomNames, 'playRoomNames result')
    
    return playRoomNames
  end
  
  def getLoginUserCountList()
    loginUserCountList = {}
    @saveDirInfo.getMaxCount.times{|i| loginUserCountList[i] = 0 }
    
    @saveDirInfo.each_with_index($loginUserInfo) do |saveFiles, index|
      if( saveFiles.size != 1 )
        logging("emptry room")
        loginUserCountList[index] = 0
        next
      end
      
      trueSaveFileName = saveFiles.first
      loginUserInfo = updateLoginUserInfo(trueSaveFileName)
      loginUserCountList[index] = loginUserInfo.size
    end
    
    return loginUserCountList
  end
  
  def getLoginUserList()
    loginUserList = {}
    @saveDirInfo.getMaxCount.times{|i| loginUserList[i] = [] }
    
    @saveDirInfo.each_with_index($loginUserInfo) do |saveFiles, index|
      if( saveFiles.size != 1 )
        logging("emptry room")
        #loginUserList[index] = []
        next
      end
      
      userNames = []
      trueSaveFileName = saveFiles.first
      loginUserInfo = updateLoginUserInfo(trueSaveFileName)
      loginUserInfo.each do |data|
        userNames << data["userName"]
      end
      
      loginUserList[index] = userNames
    end
    
    return loginUserList
  end
  
  
  def getSaveDataLastAccessTimes()
    @saveDirInfo.getSaveDataLastAccessTimes($saveFiles.values)
  end
  
  def getPlayRoomStates()
    playRoomStates = []
    
    loginUserCountList = getLoginUserCountList()
    loginUsersList = getLoginUserList()
    playRoomNames = getPlayRoomNames()
    playRoomPasswordLockStates = getPlayRoomPasswordLockStates()
    saveDataLastAccesTimes = getSaveDataLastAccessTimes()
    
    $saveDataMaxCount.times do |i|
      createdState = false
      loginUserCount = loginUserCountList[i]
      loginUsers = loginUsersList[i].join("  ")
      logging("loginUsers", loginUsers);
      playRoomName = playRoomNames[i]
      passwordLockState = (playRoomPasswordLockStates[i] ? "有り" : "--")
      timeStamp = saveDataLastAccesTimes[i]
      
      timeString = ""
      unless( timeStamp.nil? )
        timeString = "#{timeStamp.strftime('%Y/%m/%d %H:%M:%S')}"
      end
      
      playRoomState = {
        'passwordLockState' => passwordLockState,
        'index' => sprintf("%3d", i),
        'playRoomName' => playRoomName,
        'loginUserCount' => loginUserCount,
        'loginUsers' => loginUsers,
        'lastUpdateTime' => timeString,
      }
      
      playRoomStates << playRoomState
    end
    
    return playRoomStates;
  end
  
  def getLoginInfo()
    logging("getLoginInfo begin")
    
    playRoomStates = getPlayRoomStates()
    logging(playRoomStates, "playRoomStates")
    
    loginMessage = getLoginMessage()
    
    autoLoginRoom = -1
    logging(@isSampleMode, "getLoginInfo:isSampleMode")
    if( @isSampleMode )
      autoLoginRoom = Time.now.to_i
    end
    
    logging(autoLoginRoom, "autoLoginRoom")
    
    cardInfos = $card.collectCardTypeAndTypeName()
    
    result = {
      "playRoomStates" => playRoomStates,
      "loginMessage" => loginMessage,
      "autoLoginRoom" => autoLoginRoom,
      "cardInfos" => cardInfos,
      "isDiceBotOn" => $isDiceBotOn,
      "uniqueId" => Time.now.to_f.to_s,
      "refreshTimeout" => $refreshTimeout,
      "version" => $version,
    }
    
    logging(result, "result")
    logging("getLoginInfo end")
    return result
  end
  
  def getLoginMessage
    loginMessage = ""
    
    if( File.exist?( $loginMessageFile ) )
      File.readlines($loginMessageFile).each do |line|
        loginMessage << line.chomp.toutf8 << "\n";
      end
      logging(loginMessage, "loginMessage")
    else
      logging("#{$loginMessageFile} is NOT found.")
    end
    
    return loginMessage
  end
  
  def deleteOldSamplePlayRoom()
    return unless( @isSampleMode )
    
    logging("deleteOldSamplePlayRoom start")
    
    saveDataLastAccesTimes = getSaveDataLastAccessTimes()
    now = Time.new
    saveDataLastAccesTimes.each do |roomNumber, lastAccessTime|
      
      next if( roomNumber < $saveDataMaxCount )
      
      diff = (now -lastAccessTime)
      logging(diff, "diff")
      oneHour = 10 * 60 * 60
      
      next if( diff <= oneHour )
      
      logging("deleteOldSamplePlayRoom delete")
      @saveDirInfo.removeSaveDir(roomNumber)
    end
  end
  
  def createPlayRoom()
    logging('createPlayRoom begin')
    
    resultText = "OK"
    begin
      deleteOldSamplePlayRoom()
      
      jsonData = getRequestData('createPlayRoomData')
      logging(jsonData, "jsonData")
      
      createPlayRoomData = getJsonDataFromText(jsonData)
      logging(createPlayRoomData, "createPlayRoomData")
      
      playRoomName = createPlayRoomData['playRoomName']
      playRoomPassword = createPlayRoomData['playRoomPassword']
      playRoomIndex = createPlayRoomData['playRoomIndex']
      logging(playRoomName, 'playRoomName')
      logging('playRoomPassword is get')
      logging(playRoomIndex, 'playRoomIndex')
      
      @saveDirInfo.setSaveDataDirIndex(playRoomIndex)
      @saveDirInfo.createDir( @isSampleMode )
      
      salt = [rand(64),rand(64)].pack("C*").tr("\x00-\x3f","A-Za-z0-9./")
      playRoomChangedPassword = playRoomPassword.crypt(salt)
      logging(playRoomChangedPassword, 'playRoomChangedPassword')
      
      trueSaveFileName = @saveDirInfo.getTrueSaveFileName($playRoomInfo)
      changeSaveData(trueSaveFileName) do |saveData|
        saveData['playRoomName'] = playRoomName
        unless( playRoomPassword.empty? )
          saveData['playRoomChangedPassword'] = playRoomChangedPassword
        end
      end
    rescue => e
      loggingForce("exception!!!!!!!!!!!!!!");
      loggingForce($!.inspect )
      loggingForce(e.inspect )
      
      resultText = e.inspect
    end
    
    result = {
      "resultText" => resultText
    }
    logging(result, 'result')
    logging('createDir finished')
    
    return result
  end
  
  def checkRemovePlayRoom(i, ignoreLoginUser)
    unless( ignoreLoginUser )
      loginUserCountList = getLoginUserCountList()
      if( loginUserCountList[i] > 0 )
        return "userExist"
      end
    end
    
    lastAccessTimes = getSaveDataLastAccessTimes()
    logging(lastAccessTimes, "lastAccessTimes")
    lastAccessTime = lastAccessTimes[i]
    logging(lastAccessTime, "lastAccessTime")
    now = Time.now
    logging(now, "now")
    spendTimes = now - lastAccessTime
    logging(spendTimes, "spendTimes")
    logging(spendTimes / 60 / 60, "spendTimes / 60 / 60")
    if( spendTimes < $deletablePassedSeconds )
      return "プレイルームの最終アクセス時間から#{$deletablePassedSeconds}秒が経過していないため削除できません"
    end
    
    return "OK"
  end
  
  def removePlayRoom()
    removePlayRoomData = getRequestDataJsonData('removePlayRoomData')
    logging(removePlayRoomData, 'removePlayRoomData')
    
    roomNumber = removePlayRoomData['roomNumber'].to_i
    logging(roomNumber, 'roomNumber')
    
    ignoreLoginUser = removePlayRoomData['ignoreLoginUser']
    logging(ignoreLoginUser, 'ignoreLoginUser')
    
    resultText = checkRemovePlayRoom(roomNumber, ignoreLoginUser)
    if( resultText == "OK" )
      @saveDirInfo.removeSaveDir(roomNumber)
    end
    
    result = {
      "resultText" => resultText,
      "roomNumber" => roomNumber
    }
    logging(result, 'result')
    
    return result
  end
  
  def getTrueSaveFileName(fileName)
    saveFileName = @saveDirInfo.getTrueSaveFileName($saveFileTempName)
  end
  
  def save()
    logging("save() begin")
    
    lastUpdateTimes = {}
    $saveFiles.collect do |key, value|
      lastUpdateTimes[key] = 0;
    end
    logging("dummy lastUpdateTimes created")
    
    saveDataAll = {}
    getCurrentSaveData(lastUpdateTimes) do |targetSaveData, saveFileTypeName|
      saveDataAll[saveFileTypeName] = targetSaveData
      logging(saveFileTypeName, "saveFileTypeName in save")
    end
    
    logging(saveDataAll, "saveDataAll tmp")
    
    saveFileName = getNewSaveFileName()
    
    deleteOldSaveFile
    
    saveData = {}
    saveData['saveDataAll'] = saveDataAll
    text = getTextFromJsonData(saveData)
    
    createSaveFile(saveFileName, text)
    
    result = {
      "saveFileName" => saveFileName,
    }
    logging(result, "save result")
    
    logging("save() end")
    
    return result
  end
  
  def getNewSaveFileName
    now = Time.now
    saveFileName = now.strftime("DodontoF_%Y_%m%d_%H%M%S_#{now.usec}.sav")
    return File.join($saveDataTempDir, saveFileName)
  end

  def getTrueSaveFileNameAll()
    existFiles = []
    
    changeSaveData($saveFileNames) do |saveData|
      existFiles = saveData["fileNames"]
      existFiles ||= []
    end
    
    logging(existFiles, 'existFiles')
    
    return existFiles
  end
  
  
  def deleteOldSaveFile
    logging('deleteOldSaveFile begin')
    
    begin
      saveFileNames = getTrueSaveFileNameAll()
      logging(saveFileNames, 'saveFileNames')
      
      regExp = /DodontoF_[\d_]+.sav/
      
      saveFileNames.each do |saveFileName|
        logging(saveFileName, 'saveFileName')
        
        baseFileName = File.basename(saveFileName)
        logging(baseFileName, 'baseFileName')
        
        next unless(regExp === baseFileName)
        
        createdTime = File.mtime(saveFileName).to_i
        now = Time.now.to_i
        diff = ( now - createdTime )
        next if( diff < $oldSaveFileDelteSeconds )
        
        logging(saveFileName, "delete saveFileName")
        File.delete(saveFileName)
      end
    rescue => e
      loggingForce("exception!!!!!!!!!!!!!!");
      loggingForce($!.inspect )
      loggingForce(e.inspect )
    end
    logging('deleteOldSaveFile end')
  end
  
  
  def checkRoomStatus()
    checkRoomStatusData = getRequestDataJsonData('checkRoomStatusData')
    logging(checkRoomStatusData, 'checkRoomStatusData')
    
    roomNumber = checkRoomStatusData['roomNumber']
    logging(roomNumber, 'roomNumber')
    
    @saveDirInfo.setSaveDataDirIndex(roomNumber)
    dirName = @saveDirInfo.getDirName()
    
    isRoomExist = ( File.exist?(dirName) ) 
    logging(isRoomExist, 'isRoomExist')
    
    isMentenanceModeOn = false;
    playRoomName = ''
    isPasswordLocked = false
    trueSaveFileName = @saveDirInfo.getTrueSaveFileName($playRoomInfo)
    isExistPlayRoomInfo = ( File.exist?(trueSaveFileName) ) 
    if( isExistPlayRoomInfo )
      changeSaveData(trueSaveFileName) do |saveData|
        playRoomName = saveData['playRoomName']
        changedPassword = saveData['playRoomChangedPassword']
        unless( changedPassword.nil? )
          isPasswordLocked = true
        end
      end
    end
    
    logging("isPasswordLocked", isPasswordLocked);
    if( checkRoomStatusData["addParam"] == "hajimetetondaanosoraheto" ) then isPasswordLocked = false; isWelcomeMessageOn = false; isMentenanceModeOn = true; end
    
    result = {
      'isRoomExist' => isRoomExist,
      'roomName' => playRoomName,
      'roomNumber' => roomNumber,
      'isPasswordLocked' => isPasswordLocked,
      'isMentenanceModeOn' => isMentenanceModeOn,
    }
    
    return result
  end
  
  def login()
    loginData = getRequestDataJsonData('loginData')
    logging(loginData, 'loginData')
    
    roomNumber = loginData['roomNumber']
    logging(roomNumber, 'roomNumber')
    password = loginData['password']
    
    @saveDirInfo.setSaveDataDirIndex(roomNumber)
    dirName = @saveDirInfo.getDirName()
    
    resultText = ''
    isRoomExist = ( File.exist?(dirName) ) 
    unless( isRoomExist )
      resultText = "プレイルームNo.#{roomNumber}は作成されていません"
    else
      trueSaveFileName = @saveDirInfo.getTrueSaveFileName($playRoomInfo)
      changeSaveData(trueSaveFileName) do |saveData|
        changedPassword = saveData['playRoomChangedPassword']
        if( changedPassword.nil? or password.crypt(changedPassword) == changedPassword )
          resultText = "OK"
        else
          resultText = "パスワードが違います"
        end
      end
    end
    
    result = {
      'resultText' => resultText,
      'roomNumber' => roomNumber,
    }
    
    return result
  end
  
  def logout()
    logoutData = getRequestDataJsonData('logoutData')
    logging(logoutData, 'logoutData')
    
    uniqueId = logoutData['uniqueId']
    logging(uniqueId, 'uniqueId');
    
    trueSaveFileName = @saveDirInfo.getTrueSaveFileName($loginUserInfo)
    changeSaveData(trueSaveFileName) do |saveData|
      saveData.each do |existUserId, userInfo|
        logging(existUserId, "existUserId in logout check")
        logging(uniqueId, 'uniqueId in logout check')
        
        if( existUserId == uniqueId )
          userInfo['isLogout'] = true
        end
      end
      
      logging(saveData, 'saveData in logout')
    end
  end
  
  
  def checkFileSizeOnMb(fileIO, size_MB)
    unless( fileIO.instance_of?( File ) or fileIO.instance_of?( Tempfile ) )
      return ""
    end
    
    if( fileIO.stat.size < (size_MB * 1024 * 1024) )
      return ""
    end
    
    return "ファイルサイズが最大値(#{size_MB}MB)以上のためアップロードに失敗しました。"
  end
  
  def uploadReplayData()
    logging("uploadReplayData() Begin")
    
    result = {
      "resultText"=> "NG",
      "replayUrl"=> "",
    }
    
    fileIO = getRequestData('Filedata')
    logging(fileIO, 'fileIO')
    
    sizeCheckResult = checkFileSizeOnMb(fileIO, $UPLOAD_REPALY_DATA_MAX_SIZE)
    if( sizeCheckResult != "" )
      result["resultText"] = sizeCheckResult
      return result;
    end
    
    fileName = fileIO.original_filename.toutf8
    fileName = getNewFileName(fileName)
    saveDir = $replayDataUploadDir
    replayDataFileName = File.join(saveDir, fileName)
    baseUrl = getRequestData('baseUrl');
    replayUrl = baseUrl + "?replay=" + CGI.escape(replayDataFileName)
    
    replayDataName = getRequestData('replayDataName');
    setReplayDataInfo(replayDataName, replayUrl)
    
    open(replayDataFileName, "w+") do |file|
      file.binmode
      file.write(fileIO.read)
    end
    
    result["resultText"] = "OK"
    result["replayUrl"] = replayUrl
    result["replayDataList"] = getReplayDataList() #[{"title"=>x, "url"=>y}]
    
    logging(result, "load result")
    logging("uploadReplayData() End")
    
    return result
  end
  
  def getReplayDataList
    replayDataList = nil
    
    changeSaveData( getReplayDataInfoFileName() ) do |saveData|
      titleInfos = saveData['titleInfos']
      replayDataList = titleInfos
    end
    
    return replayDataList
  end
  
  def getReplayDataInfoFileName
    infoFileName = File.join($replayDataUploadDir, 'replayDataInfo.json')
    return infoFileName
  end
  

  #getImageInfoFileName() ) do |saveData|
  def setReplayDataInfo(title, url)
    changeSaveData( getReplayDataInfoFileName() ) do |saveData|
      saveData['titleInfos'] ||= []
      titleInfos = saveData['titleInfos']
      titleInfos << {
        "title" => title,
        "url" => url,
      }
    end
  end
  
  def load()
    logging("saveData load() Begin")
    
    fileIO = getRequestData('Filedata')
    logging(fileIO, 'fileIO')
    
    jsonDataString = fileIO.read
    logging(jsonDataString, 'jsonDataString')
    
    jsonData = getJsonDataFromText(jsonDataString)
    logging(jsonData, 'jsonData')
    saveDataAll = jsonData['saveDataAll']
    
    targets = getRequestData('targets');
    
    if( targets.nil? ) 
      loadSaveFileDataAll(saveDataAll)
    else
      loadSaveFileDataFilterByTargets(saveDataAll, targets)
    end
    
    result = {
      "resultText"=> "OK"
    }
    logging(result, "load result")
    
    return result
  end
  
  def getLoadData(saveDataAll, fileType, key, defaultValue)
    saveFileData = saveDataAll[fileType]
    return defaultValue if(saveFileData.nil?)
    
    data = saveFileData[key]
    return defaultValue if(data.nil?)
    
    return data.clone
  end
  
  def loadCharacterDataList(saveDataAll, type)
    characterDataList = getLoadData(saveDataAll, 'characters', 'characters', [])
    logging(characterDataList, "characterDataList")
    
    characterDataList = characterDataList.delete_if{|i| (i["type"] != type)}
    addCharacterData( characterDataList )
  end
  
  def loadSaveFileDataFilterByTargets(saveDataAll, targets)
    targets.each do |target|
      case target
      when "map"
        mapData = getLoadData(saveDataAll, 'map', 'mapData', {})
        changeMapSaveData(mapData)
      when "characterData", "mapMask", "mapMarker", "magicRangeMarker", "magicRangeMarkerDD4th"
        loadCharacterDataList(saveDataAll, target)
      when "standingGraphicInfos"
        effects = getLoadData(saveDataAll, 'effects', 'effects', [])
        effects = effects.delete_if{|i| (i["type"] != target)}
        loggingForce(effects, "standingGraphicInfos effects");
        addEffectData(effects)
      when "cutIn"
        effects = getLoadData(saveDataAll, 'effects', 'effects', [])
        effects = effects.delete_if{|i| (i["type"] != nil)}
        addEffectData(effects)
      end
    end
  end
  
  def loadSaveFileDataAll(saveDataAll)
    @saveFiles.each do |fileTypeName, trueSaveFileName|
      
      saveDataTmp = saveDataAll[fileTypeName]
      saveDataTmp ||= {}
      
      changeSaveData(trueSaveFileName) do |saveDataCurrent|
        saveDataCurrent.clear
        saveDataTmp.each do |key, value|
          saveDataCurrent[key] = value
        end
      end
    end
  end
  
  def uploadImageFile()
    logging("uploadImageFile load Begin")
    
    result = {
      "resultText"=> "OK"
    }
    
    fileIO = getRequestData('Filedata')
    logging(fileIO, 'fileIO')
    
    sizeCheckResult = checkFileSizeOnMb(fileIO, $UPLOAD_IMAGE_MAX_SIZE)
    if( sizeCheckResult != "" )
      result["resultText"] = sizeCheckResult
      return result;
    end
    
    saveDir = $imageUploadDir
    
    fileName = fileIO.original_filename.toutf8
    logging(fileName, "fileName original")
    fileName = getNewFileName(fileName, "img")
    logging(fileName, "fileName")
    
    uploadImageFileName = File.join(saveDir, fileName)
    logging(uploadImageFileName, "uploadImageFileName")
    
    open(uploadImageFileName, "w+") do |file|
      file.binmode
      file.write(fileIO.read)
    end
    
    tagInfo = getRequestData('tagInfo')
    changeImageTagsLocal(uploadImageFileName, tagInfo)
    
    logging(result, "uploadImageFile result")
    
    return result
  end
  
  def getNewFileName(fileName, preFix = "")
    @newFileNameIndex ||= 0
    extName = ""
    if( /\.(.+)$/ =~ fileName )
      extName = $1
    end
    logging(extName, "extName")
    
    result = preFix + Time.now.to_f.to_s + "_" + @newFileNameIndex.to_s + "." + extName
    
    return result.untaint
  end
  
  def deleteImage()
    logging("deleteImage begin")
    
    jsonData = getRequestData('imageData')
    logging(jsonData, "jsonData")
    
    imageData = getJsonDataFromText(jsonData)
    logging(imageData, "imageData")
    
    imageUrlList = imageData['imageUrlList']
    logging(imageUrlList, "imageUrlList")
    
    imageFiles = getAllImageFileNameFromTagInfoFile()
    logging(imageFiles, "imageFiles")
    
    imageUrlFileName = $imageUrlText
    logging(imageUrlFileName, "imageUrlFileName")
    
    deleteCount = 0
    resultText = ""
    imageUrlList.each do |imageUrl|
      deleteResult = deleteImageTags(imageUrl)
      deleteResult ||= deleteTargetImageUrl(imageUrl, imageFiles, imageUrlFileName)
      
      if( deleteResult )
        deleteCount += 1
      else
        warningMessage = "不正な操作です。あなたが削除しようとしたファイル(#{imageUrl})はイメージファイルではありません。"
        loggingForce(warningMessage)
        resultText += warningMessage
      end
    end
    
    resultText += "#{deleteCount}個のファイルを削除しました。"
    result = {"resultText" => resultText}
    logging(result, "result")
    
    logging("deleteImage end")
    return result
  end
  
  def deleteTargetImageUrl(imageUrl, imageFiles, imageUrlFileName)
    logging(imageUrl, "deleteTargetImageUrl(imageUrl)")
    
    if( imageFiles.include?(imageUrl) )
      if( File.exist?(imageUrl) )
        File.delete(imageUrl);
        return true
      end
    end
    
    locker = getSaveFileLock(imageUrlFileName)
    locker.lock do 
      lines = File.readlines(imageUrlFileName)
      logging(lines, "lines")
      
      deleteResult = lines.reject!{|i| i.chomp == imageUrl }
      
      unless( deleteResult )
        return false
      end
      
      logging(lines, "lines deleted")
      File.open(imageUrlFileName, "w+") do |file|
        file.write(lines);
      end
    end
    
    return true
  end
  
  def uploadImageUrl()
    logging("uploadImageUrl begin")
    
    jsonData = getRequestData('imageData')
    logging(jsonData, "jsonData")
    
    imageData = getJsonDataFromText(jsonData)
    logging(imageData, "imageData")
    
    imageUrl = imageData['imageUrl']
    logging(imageUrl, "imageUrl")
    
    imageUrlFileName = $imageUrlText
    logging(imageUrlFileName, "imageUrlFileName")
    
    resultText = "画像URLのアップロードに失敗しました。"
    locker = getSaveFileLock(imageUrlFileName)
    locker.lock do 
      alreadyExistUrls = File.readlines(imageUrlFileName).collect{|i| i.chomp }
      if( alreadyExistUrls.include?(imageUrl) )
        resultText = "すでに登録済みの画像URLです。"
      else
        File.open(imageUrlFileName, "a+") do |file|
          file.write(imageUrl + "\n");
        end
        resultText = "画像URLのアップロードに成功しました。"
      end
    end
    
    tagInfo = imageData['tagInfo']
    logging(tagInfo, 'uploadImageUrl.tagInfo')
    changeImageTagsLocal(imageUrl, tagInfo)
    
    logging("uploadImageUrl end")
    
    result = {"resultText" => resultText}
    return result
  end
  
  
  
  def getGraveyardCharacterData()
    logging("getGraveyardInfo start.")
    result = []
    
    changeSaveData(@saveFiles['characters']) do |saveData|
      graveyard = saveData['graveyard']
      graveyard ||= []
      
      result = graveyard.reverse;
    end
    
    return result;
  end
  
  def getImageList()
    logging("getImageList start.")
    imageDataJsonString = getRequestData('imageData')
    logging(imageDataJsonString, "imageDataJsonString")
    
    imageData = getJsonDataFromText(imageDataJsonString)
    logging(imageData, "imageData")
    
    imageList = getAllImageFileNameFromTagInfoFile()
    logging(imageList, "imageList all result")
    
    addTextsCharacterImageList(imageList, $imageUrlText)
    
    imageList.delete_if{|i| (/\.txt$/===i)}
    imageList.delete_if{|i| (/\.lock$/===i)}
    imageList.delete_if{|i| (/\.json$/===i)}
    imageList.delete_if{|i| (/\.json~$/===i)}
    imageList.delete_if{|i| (/^.svn$/===i)}
    imageList.delete_if{|i| (/.db$/===i)}
    imageList.sort!
    
    imageListData = {
      "imageList" => imageList,
      "imageDir" => $imageUploadDir,
    }
    
    return imageListData
  end
  
  def deleteOldChatMessageData(chatMessageDataLog)
    now = Time.now.to_f
    
    chatMessageDataLog.delete_if do |chatMessageData|
      writtenTime, chatMessage, *dummy = chatMessageData
      timeDiff = now - writtenTime
      
      ( timeDiff > ($oldMessageTimeout) )
    end
  end
  
  def sendChatMessageMany
    100.times{ sendChatMessage }
  end
  
  def sendChatMessage
    chatMessageData = nil
    
    changeSaveData(@saveFiles['chatMessageDataLog']) do |saveData|
      
      jsonData = getRequestData('chatMessageData')
      logging(jsonData, "jsonData")
      chatMessageData = getJsonDataFromText(jsonData)
      logging(chatMessageData, "chatMessageData")
      
      chatMessageDataLog = saveData['chatMessageDataLog']
      chatMessageDataLog ||= []
      
      deleteOldChatMessageData(chatMessageDataLog);
      
      begin
        now = Time.now.to_f
        chatMessageData = [now, chatMessageData]
        
      rescue => e
        loggingForce("exception!!!!!!!!!!!!!!");
        loggingForce($!.inspect )
        loggingForce(e.inspect )
      end
      
      chatMessageDataLog.push(chatMessageData)
      chatMessageDataLog.sort!
      logging(chatMessageDataLog, "chatMessageDataLog")
      saveData['chatMessageDataLog'] = chatMessageDataLog
      logging(saveData['chatMessageDataLog'], "saveData['chatMessageDataLog']");
    end
    
    if( $IS_SAVE_LONG_CHAT_LOG )
      saveAllChatMessage(chatMessageData)
    end
  end
  
  def saveAllChatMessage(chatMessageData)
    if( chatMessageData.nil? )
      return
    end
    
    saveFileName = @saveDirInfo.getTrueSaveFileName($chatMessageDataLogAll)
    
    locker = getSaveFileLock(saveFileName)
    locker.lock do 
      
      lines = []
      if( File.exist?(saveFileName) )
        lines = File.readlines(saveFileName)
      end
      lines << getTextFromJsonData(chatMessageData)
      lines << "\n"
      
      while( lines.size > $chatMessageDataLogAllLineMax )
        lines.shift
      end
      
      File.open(saveFileName, 'w+') do |file|
        file.write(lines)
      end
    end
    
  end
  
  def changeMap
    mapData = getRequestDataJsonData('mapData')
    logging(mapData, "mapData")
    
    changeMapSaveData(mapData)
  end
  
  def changeMapSaveData(mapData)
    logging("changeMap start.")
    
    changeSaveData(@saveFiles['map']) do |saveData|
      
      saveData['mapData'] ||= {}
      logging(saveData['mapData'], "saveData['mapData']")
      
      saveData['mapData'] = mapData
    end
  end
  
  def addEffect
    effectData = getRequestDataJsonData('effectData')
    effectDataList = [effectData]
    addEffectData(effectDataList)
  end
  
  def addEffectData(effectDataList)
    changeSaveData(@saveFiles['effects']) do |saveData|
      saveData['effects'] ||= []
      effects = saveData['effects']
      
      effectDataList.each do |effectData|
        effectData['effectId'] = createCharacterImgId("effects_")
        effects << effectData
      end
    end
  end
  
  def changeEffect
    changeSaveData(@saveFiles['effects']) do |saveData|
      effectData = getRequestDataJsonData('effectData')
      targetCutInId = effectData['effectId']
      
      saveData['effects'] ||= []
      effects = saveData['effects']
      
      findIndex = -1
      effects.each_with_index do |i, index|
        if( targetCutInId == i['effectId'] )
          findIndex = index
        end
      end
      
      if( findIndex == -1 )
        return
      end
      
      effects[findIndex] = effectData
    end
  end
  
  def removeEffect
    changeSaveData(@saveFiles['effects']) do |saveData|
      effectData = getRequestDataJsonData('effectData')
      effectId = effectData['effectId']
      saveData['effects'] ||= []
      effects = saveData['effects']
      effects.delete_if{|i| (effectId == i['effectId'])}
    end
  end
  
  
  
  def getImageInfoFileName
    imageInfoFileName = File.join($imageUploadDir, 'imageInfo.json')
    
    logging(imageInfoFileName, 'imageInfoFileName')
    
    return imageInfoFileName
  end
  
  def changeImageTags
    effectData = getRequestDataJsonData('tagsData')
    source = effectData['source']
    tagInfo = effectData['tagInfo']
    
    changeImageTagsLocal(source, tagInfo)
  end
  
  def getAllImageFileNameFromTagInfoFile()
    imageFileNames = []
    
    changeSaveData( getImageInfoFileName() ) do |saveData|
      imageTags = saveData['imageTags']
      imageTags ||= {}
      imageFileNames = imageTags.keys
    end
    
    return imageFileNames
  end
  
  def changeImageTagsLocal(source, tagInfo = nil)
    if( tagInfo.nil? )
      return
    end
    
    changeSaveData( getImageInfoFileName() ) do |saveData|
      saveData['imageTags'] ||= {}
      imageTags = saveData['imageTags']
      
      imageTags[source] = tagInfo
    end
  end
  
  def deleteImageTags(source)
    result = nil
    
    changeSaveData( getImageInfoFileName() ) do |saveData|
      imageTags = saveData['imageTags']
      result ||= imageTags.delete(source)
    end
    
    return result
  end
  
  def getImageTags
    logging('getImageTags start')
    imageTags = nil
    
    changeSaveData( getImageInfoFileName() ) do |saveData|
      imageTags = saveData['imageTags']
    end
    
    imageTags ||= {}
    logging(imageTags, 'getImageTags imageTags')
    
    return imageTags
  end
  
  def createCharacterImgId(prefix = "character_")
    @imgIdIndex ||= 0;
    @imgIdIndex += 1;
    
    return (prefix + Time.now.to_f.to_s + "_" + @imgIdIndex.to_s);
  end
  
  def addCharacter
    jsCharacterData = getRequestData('characterData')
    characterData = getJsonDataFromText(jsCharacterData)
    characterDataList = [characterData]
    
    addCharacterData( characterDataList )
  end
  
  def isAlreadyExistCharacter?(characters, characterData)
    return false if( characterData['name'].nil? )
    return false if( characterData['name'].empty? )
    
    alreadyExist = characters.find do |i|
      (i['imgId'] == characterData['imgId']) or
        (i['name'] == characterData['name'])
    end
    
    return false if( alreadyExist.nil? )
    
    logging("target characterData is already exist. no creation.", "isAlreadyExistCharacter?")
    return characterData['name']
  end
  
  def addCharacterData(characterDataList)
    result = {
      "addFailedCharacterNames" => []
    }
    
    changeSaveData(@saveFiles['characters']) do |saveData|
      saveData['characters'] ||= []
      characters = saveData['characters']
      
      characterDataList.each do |characterData|
        logging(characterData, "characterData")
        
        characterData['imgId'] = createCharacterImgId()
        
        failedName = isAlreadyExistCharacter?(characters, characterData)
        if( failedName )
          result["addFailedCharacterNames"] << failedName
          next
        end
        
        logging("add characterData to characters")
        characters << characterData
      end
    end
    
    return result
  end
  
  def changeCharacterData
    changeSaveData(@saveFiles['characters']) do |saveData|
      logging("changeCharacterData called")
      
      jsCharacterData = getRequestData('characterData')
      ddCharacter = getJsonDataFromText(jsCharacterData)
      logging(ddCharacter.inspect, "ddCharacter")
      
      characters = saveData['characters']
      
      index = nil
      characters.each_with_index  do |item, targetIndex|
        if(item['imgId'] == ddCharacter['imgId'])
          index = targetIndex
          break;
        end
      end
      
      if( index.nil? )
        logging("invalid character name")
        return
      end
      
      unless( ddCharacter['name'].nil? or ddCharacter['name'].empty? )
        alreadyExist = characters.find do |character|
          ( (character['name'] == ddCharacter['name']) and
              (character['imgId'] != ddCharacter['imgId']) )
        end
        
        if( alreadyExist ) 
          logging("same name character alread exist");
          return;
        end
      end
      
      logging(ddCharacter.inspect, "character data change")
      characters[index] = ddCharacter
    end
  end
  
  def getCardType
    "Card"
  end
  
  def getCardMountType
    "CardMount"
  end
  
  def getCardTrushMountType
    "CardTrushMount"
  end
  
  def getRotation(isUpDown)
    rotation = 0
    
    if( isUpDown )
      if( rand(2) == 0 )
        rotation = 180
      end
    end
    
    return rotation
  end
  
  def getCardData(isText, imageName, imageNameBack, mountName, isUpDown = false, canDelete = false)
    
    return {
      "imageName" => imageName,
      "imageNameBack" => imageNameBack,
      "isBack" => true,
      "rotation" => getRotation(isUpDown),
      "isText" => isText,
      "isOpen" => false,
      "owner" => "",
      "ownerName" => "",
      "mountName" => mountName,
      "canDelete" => canDelete,
      
      "name" => "",
      "imgId" =>  createCharacterImgId(),
      "type" => getCardType(),
      "x" => 0,
      "y" => 0,
      "draggable" => true,
    }
  end
  
  def initCards
    logging("initCards Begin");
    
    clearCharacterByTypeLocal(getCardType)
    clearCharacterByTypeLocal(getCardMountType)
    clearCharacterByTypeLocal(getCardTrushMountType)
    
    jsData = getRequestData('initCardsData')
    initCardData = getJsonDataFromText(jsData)
    cardTypeInfos = initCardData['cardTypeInfos']
    logging(cardTypeInfos, "cardTypeInfos")
    
    changeSaveData(@saveFiles['characters']) do |saveData|
      saveData['cardTrushMount'] = {}
      
      saveData['cardMount'] = {}
      cardMounts = saveData['cardMount']
      
      characters = saveData['characters']
      logging(characters, "initCards saveData.characters")
      
      cardTypeInfos.each_with_index do |cardTypeInfo, index|
        mountName = cardTypeInfo['mountName']
        logging(mountName, "initCards mountName")
        
        cardsListFileName = $card.getCardFileName(mountName);
        logging(cardsListFileName, "initCards cardsListFileName");
        
        cardsList = []
        File.readlines(cardsListFileName).each_with_index  do |i, lineIndex|
          cardsList << i.chomp.toutf8
        end
        
        logging(cardsList, "initCards cardsList")
        
        cardData = cardsList.shift.split(/,/)
        isText = (cardData.shift == "text")
        isUpDown = (cardData.shift == "upDown")
        logging("isUpDown", isUpDown)
        imageNameBack = cardsList.shift
        
        cardsList = getInitCardSet(cardsList, cardTypeInfo)
        cardMounts[mountName] = getInitedCardMount(cardsList, mountName, isText, isUpDown, imageNameBack)
        
        cardMountData = getCardMountData(cardMounts, isText, imageNameBack, mountName, index, isUpDown)
        characters << cardMountData
        
        cardTrushMountData = getCardTrushMountData(isText, mountName, index)
        characters << cardTrushMountData
      end
    end
    
    logging("initCards End");
  end
  
  
  def getInitedCardMount(cardsList, mountName, isText, isUpDown, imageNameBack)
    cardMount = []
    
    cardsList.each do |imageName|
      if( /^###Back###(.+)/ === imageName  )
        imageNameBack = $1
        next
      end
      
      logging(imageName, "initCards imageName")
      cardData = getCardData(isText, imageName, imageNameBack, mountName, isUpDown)
      cardMount << cardData
    end
    
    return cardMount.sort_by{rand}
  end
  
  
  def addCard()
    loggingForce("addCard begin");
    
    jsAddCardData = getRequestData('addCardData')
    addCardData = getJsonDataFromText(jsAddCardData)
    
    isText = addCardData['isText']
    imageName = addCardData['imageName']
    imageNameBack = addCardData['imageNameBack']
    mountName = addCardData['mountName']
    isUpDown = addCardData['isUpDown']
    canDelete = addCardData['canDelete']
    
    changeSaveData(@saveFiles['characters']) do |saveData|
      cardData = getCardData(isText, imageName, imageNameBack, mountName, isUpDown, canDelete)
      cardData["x"] = addCardData['x']
      cardData["y"] = addCardData['y']
      
      characters = saveData['characters']
      characters << cardData
    end
    
    loggingForce("addCard end");
    
  end
  
  #トランプのジョーカー枚数、使用デッキ数の指定
  def getInitCardSet(cardsList, cardTypeInfo)
    useLineCount = cardTypeInfo['useLineCount']
    useLineCount ||= cardsList.size
    logging(useLineCount, 'useLineCount')
    
    deckCount = cardTypeInfo['deckCount']
    deckCount ||= 1
    logging(deckCount, 'deckCount')
    
    
    cardsListTmp = []
    deckCount.to_i.times do
      cardsListTmp += cardsList[0...useLineCount]
    end
    
    return cardsListTmp
  end
  
  
  def getCardMountData(cardMount, isText, imageNameBack, mountName, index, isUpDown)
    cardMountData = getCardData(isText, imageNameBack, imageNameBack, mountName)
    
    cardMountData['type'] = getCardMountType
    setCardCountAndBackImage(cardMountData, cardMount[mountName]);
    cardMountData['mountName'] = mountName
    cardMountData['isUpDown'] = isUpDown
    cardMountData['x'] = 30 + index * 150
    cardMountData['y'] = 30;
    
    return cardMountData
  end
  
  def getCardTrushMountData(isText, mountName, index)
    cardTitle = $card.getCardTitleName(mountName);
    trushPrintName = "<font size=\"40\">#{cardTitle}用<br>カード捨て場</font>"
    
    isText = true
    
    cardTrushMountData = getCardData(isText, trushPrintName, trushPrintName, mountName)
    
    cardTrushMountData['type'] = getCardTrushMountType
    cardTrushMountData['cardCount'] = 0
    cardTrushMountData['mountName'] = mountName
    cardTrushMountData['x'] = 30 + index * 150
    cardTrushMountData['y'] = 230;
    
    return cardTrushMountData
  end
  
  
  def returnCard
    logging("returnCard Begin");
    
    jsData = getRequestData('data')
    params = getJsonDataFromText(jsData)
    
    mountName = params['mountName']
    logging(mountName, "mountName")
    
    changeSaveData(@saveFiles['characters']) do |saveData|
      saveData['cardTrushMount'] ||= {}
      trushMount = saveData['cardTrushMount']
      logging(trushMount, 'trushMount')
      
      trushMount[mountName] ||= []
      trushCards = trushMount[mountName]
      
      cardData = trushCards.pop
      logging(cardData, "cardData")
      if( cardData.nil? )
        logging("returnCard trushCards is empty. END.")
        return
      end
      
      cardData['x'] = params['x'] + 150
      cardData['y'] = params['y'] + 10
      logging('returned cardData', cardData)
      
      characters = saveData['characters']
      characters.push( cardData )
      
      trushMountData = characters.find{|i| i['imgId'] === params['imgId'] }
      logging(trushMountData, "returnCard trushMountData")
      return if( trushMountData.nil?) 
      
      trushMountData['cardCount'] = trushCards.size
    end
    
    logging("returnCard End");
  end
  
  def drawCard
    logging("drawCard Begin")
    
    jsData = getRequestData('drawCardData')
    drawCardData = getJsonDataFromText(jsData)
    logging(drawCardData, 'drawCardData')
    
    mountName = drawCardData['mountName']
    logging(mountName, 'mountName')
    
    changeSaveData(@saveFiles['characters']) do |saveData|
      cardMount = saveData['cardMount']
      cardMount ||= {}
      
      cards = cardMount[mountName]
      cards ||= []
      
      cardData = cards.pop
      if( cardData.nil? )
        return
      end
      
      cardData['x'] = drawCardData['x']
      cardData['y'] = drawCardData['y']
      
      isOpen = drawCardData['isOpen']
      cardData['isOpen'] = isOpen
      cardData['isBack'] = false
      cardData['owner'] = drawCardData['owner']
      cardData['ownerName'] = drawCardData['ownerName']
      
      saveData['characters'] ||= []
      characters = saveData['characters']
      characters << cardData
      
      cardMountData = characters.find{|i| i['imgId'] === drawCardData['imgId'] }
      if( cardMountData.nil?) 
        return
      end
      
      logging(cards.size, 'cardMount[mountName].size')
      setCardCountAndBackImage(cardMountData, cards)
    end
    
    logging("drawCard End")
  end
  
  def setCardCountAndBackImage(cardMountData, cards)
    cardMountData['cardCount'] = cards.size
    
    card = cards.last
    return if( card.nil? )
    
    image = card["imageNameBack"];
    return if( image.nil? ) 
    
    cardMountData["imageNameBack"] = image;
  end
  
  def dumpTrushCards
    logging("dumpTrushCards Begin")
    
    jsData = getRequestData('data')
    dumpTrushCardsData = getJsonDataFromText(jsData)
    logging(dumpTrushCardsData, 'dumpTrushCardsData')
    
    mountName = dumpTrushCardsData['mountName']
    logging(mountName, 'mountName')
    
    changeSaveData(@saveFiles['characters']) do |saveData|
      saveData['cardTrushMount'] ||= {}
      trushMount = saveData['cardTrushMount']
      logging(trushMount, 'trushMount')
      
      trushMount[mountName] ||= []
      trushCards = trushMount[mountName]
      
      characters = saveData['characters']
      
      dumpedCardId = dumpTrushCardsData['dumpedCardId']
      logging( dumpedCardId, "dumpedCardId")
      
      logging(characters.size, "characters.size before")
      cardData = deleteFindOne(characters) {|i| i['imgId'] === dumpedCardId }
      trushCards << cardData
      logging(characters.size, "characters.size after")
      
      trushMountCharacter = characters.find{|i| i['imgId'] === dumpTrushCardsData['trushMountId'] }
      if( trushMountCharacter.nil?) 
        return
      end
      
      logging(trushMount, 'trushMount')
      logging(mountName, 'mountName')
      logging(trushMount[mountName], 'trushMount[mountName]')
      logging(trushMount[mountName].size, 'trushMount[mountName].size')
      trushMountCharacter['cardCount'] = trushMount[mountName].size
    end
    
    logging("dumpTrushCards End")
  end
  
  def deleteFindOne(array)
    findIndex = nil
    array.each_with_index do |i, index|
      if( yield(i) )
        findIndex = index
      end
    end
    
    if( findIndex.nil? )
      throw Exception.new("deleteFindOne target is NOT found inspect:" ) #+ array.inspect)
    end
    
    logging(array.size, "array.size before")
    item = array.delete_at(findIndex)
    logging(array.size, "array.size before")
    return item
  end
  
  def shuffleCards
    logging("shuffleCard Begin")
    
    jsData = getRequestData('data')
    shuffleCardData = getJsonDataFromText(jsData)
    logging(shuffleCardData, 'shuffleCardData')
    
    imgId = shuffleCardData['imgId']
    logging(imgId, 'imgId')
    
    mountName = shuffleCardData['mountName']
    logging(imgId, 'mountName')
    
    changeSaveData(@saveFiles['characters']) do |saveData|
      saveData['cardTrushMount'] ||= {}
      trushMount = saveData['cardTrushMount']
      
      trushMount[mountName] ||= []
      trushCards = trushMount[mountName]
      
      saveData['cardMount'] ||= {}
      cardMount = saveData['cardMount']
      cardMount[mountName] ||= []
      mountCards = cardMount[mountName]
      
      logging(trushCards, 'shuffleCards trushCards');
      while( trushCards.size > 0 )
        cardData = trushCards.pop
        logging(cardData, "shuffleCards cardData")
        cardData['isOpen'] = false
        cardData['isBack'] = true
        cardData['owner'] = ""
        cardData['ownerName'] = ""
        
        mountCards << cardData
      end
      
      characters = saveData['characters']
      
      trushMountData = characters.find{|i| i['imgId'] === shuffleCardData['imgId'] }
      logging(trushMountData, "dumpTrushCards trushMountData")
      return if( trushMountData.nil?) 
      trushMountData['cardCount'] = trushCards.size

      cardMountData = characters.find do |i| 
        ((i['type'] === getCardMountType) && (i['mountName'] == mountName))
      end
      logging(cardMountData, "dumpTrushCards cardMountData")
      return if( cardMountData.nil?) 
      
      isShuffle = shuffleCardData['isShuffle']
      if( isShuffle )
        isUpDown = cardMountData['isUpDown']
        mountCards = getShuffledMount(mountCards, isUpDown)
      end
      
      cardMount[mountName] = mountCards
      saveData['cardMount'] = cardMount
      
      setCardCountAndBackImage(cardMountData, mountCards)
    end
    
    logging("drawCard End")
  end
  
  def getShuffledMount(mountCards, isUpDown)
    mountCards = mountCards.sort_by{rand}
    mountCards.each do |i|
      i["rotation"] = getRotation(isUpDown)
    end
    
    return mountCards
  end
  
  def clearCharacterByType
    jsData = getRequestData('clearData')
    clearData = getJsonDataFromText(jsData)
    logging(clearData, 'clearData')
    
    targetTypes = clearData['types']
    logging(targetTypes, 'targetTypes')
    
    targetTypes.each do |targetType|
      clearCharacterByTypeLocal(targetType)
    end
  end
  
  def clearCharacterByTypeLocal(targetType)
    logging(targetType, "clearCharacterByTypeLocal targetType")
    
    changeSaveData(@saveFiles['characters']) do |saveData|
      characters = saveData['characters']
      
      characters.delete_if do |i|
        (i['type'] == targetType)
      end
    end
    
    logging("clearCharacterByTypeLocal End")
  end
  
  def removeCharacter
    changeSaveData(@saveFiles['characters']) do |saveData|
      
      jsRemoveCharacterData = getRequestData('removeCharacterData')
      removeCharacterData = getJsonDataFromText(jsRemoveCharacterData)
      logging(removeCharacterData, "removeCharacterData")
      
      removeCharacterId = removeCharacterData['imgId']
      logging(removeCharacterId, "removeCharacterId")
      isGotoGraveyard = removeCharacterData['isGotoGraveyard']
      logging(isGotoGraveyard, "isGotoGraveyard")
      
      characters = saveData['characters']
      
      deleteResultCharacters = characters.delete_if do |i|
        deleted = (i['imgId'] == removeCharacterId)
        
        if( deleted and isGotoGraveyard )
          moveCharacterToGraveyard(i, saveData)
        end
        
        deleted
      end
      
      logging(deleteResultCharacters, "character deleted result")
    end
  end
  
  def moveCharacterToGraveyard(character, saveData)
    saveData['graveyard'] ||= []
    graveyard = saveData['graveyard']
    
    graveyard << character
    
    while(graveyard.size > $graveyardLimit)
      graveyard.shift
    end
  end
  

  def resurrectCharacter
    changeSaveData(@saveFiles['characters']) do |saveData|
      
      jsResurrectCharacterData = getRequestData('resurrectCharacterData')
      resurrectCharacterData = getJsonDataFromText(jsResurrectCharacterData)
      logging(resurrectCharacterData, "resurrectCharacterData")
      
      resurrectCharacterId = resurrectCharacterData['imgId']
      logging(resurrectCharacterId, "resurrectCharacterId")
      
      graveyard = saveData['graveyard']
      graveyard ||= []
      
      index = nil
      graveyard.each_with_index do |i, targetIndex|
        logging(i, "characterData");
        if( i['imgId'] == resurrectCharacterId )
          index = targetIndex
        end
      end
      
      logging(index, "resurrected index:")
      
      if( index )
        characterData = graveyard.delete_at(index)
        logging(characterData, "resurrectedCharacterData");
        
        saveData['characters'] ||= []
        characters = saveData['characters']
        characters << characterData
      end
    end
  end
  
  def changeRoundTime
    changeSaveData(@saveFiles['time']) do |saveData|
      jsRoundTimeData = getRequestData('roundTimeData')
      roundTimeData = getJsonDataFromText(jsRoundTimeData)
      
      saveData['roundTimeData'] = roundTimeData;
    end
  end
  
  def moveCharacter
    changeSaveData(@saveFiles['characters']) do |saveData|

      jsCharacterData = getRequestData('characterData')
      targetCharacter = getJsonDataFromText(jsCharacterData)
      logging(targetCharacter, "moveCharacter() targetCharacter")
      
      imgId = targetCharacter['imgId']
      x = targetCharacter['x']
      y = targetCharacter['y']
      
      logging(imgId, "character.imgId")
      
      characters = saveData['characters']
      index = -1
      characters.each_with_index do |data, i|
        if( data['imgId'] == imgId )
          index = i
        end
      end
      
      if( index == -1 )
        logging("character is no exist", "moveCharacter")
        return
      end
      
      logging(characters, "characters")
      character = characters.delete_at(index)
      characters << character
      
      character['x'] = x
      character['y'] = y
    end
  end
  
  def getSaveFileTimeStamp(saveFileName)
    unless( File.exist?(saveFileName) ) 
      return 0
    end
    
    timeStamp = File.mtime(saveFileName).to_i
    return timeStamp
  end
  
  def isSaveFileChanged(lastUpdateTime, saveFileName)
    saveFileTimeStamp = getSaveFileTimeStamp(saveFileName);
    changed = (saveFileTimeStamp > lastUpdateTime)
    return changed
  end
  
  def getResponse
    jsonDataForResponse = analyzeCommand
    #escapeHtmlDataFromJson(jsonDataForResponse)
    
    return getTextFromJsonData(jsonDataForResponse)
  end
end

if( $0 === __FILE__ )
  logging("========================================>CGI begin.")
  
  text = "empty"
  begin
    cgi = CGI.new
    server = DodontoFServer.new(cgi, cgi.content_type)
    result = server.getResponse
    
    header = "";
    header << "Content-Type: text/plain\ncharset: utf-8\n\n";
    logging(header, "header")
    
    #result = "###DodontoF BEGIN###" + result + "###DodontoF END###";
    result = "#D@EM>#" + result + "#<D@EM#";
    text = header + result
  rescue => e
    errorMessage = ""
    errorMessage << "e.to_s : " << e.to_s << "\n"
    errorMessage << "e.inspect : " << e.inspect << "\n"
    errorMessage << "$@ : " << $@.join("\n") << "\n"
    errorMessage << "$! : " << $!.to_s << "\n"
    loggingForce(errorMessage, "errorMessage")
    
    text = cgi.header({
                        'type' => 'text/html',
                      })
    text << "\n= ERROR ====================\n"
    text << errorMessage
    text << "============================\n"
  end
  
  logging(text, "CGI respons string")
  print(text)
  
  logging("========================================>CGI end.")
end

