//--*-coding:utf-8-*--

package {
    
    import flash.events.Event;
    import flash.events.MouseEvent;
    import flash.events.TimerEvent;
    import flash.utils.Timer;
    import mx.events.FlexEvent;
    import mx.controls.Text;
    import mx.controls.TextArea;
    import mx.controls.Image;
    import mx.containers.HBox;
    import mx.controls.Button;
    import mx.controls.Alert;
    import mx.events.CloseEvent;
    
    public class ChatMessageTrader {
        
        private var cutInList:Array = new Array();
        private var cutInDelayInfoList:Array = new Array();
        private var chatWindow:ChatWindow;
        private var addMessageLines:String = "";
        
        private var sendedChatMessageData:Object = new Object();
        
        private var localMessageUniqueId:String = "localMessageUniqueId";
        private var systemMessageSenderName:String = "どどんとふ";
        private var systemMessageColor:String = "00AA00";
        private var thisObj:ChatMessageTrader;
        
        private var loadingLimitSeconds:int = 15;
        
        private var voter:Voter;
        
        
        public function ChatMessageTrader(chatWindow_:ChatWindow) {
            thisObj = this;
            
            chatWindow = chatWindow_;
            cutInList = [new CutInMovie(), new CutInImage()];
            
            voter  = new Voter(this);
        }
        
        
        public function clearAddMessageLines():void {
            addMessageLines = "";
        }
        
        public function pushAddMessageLines(line:String):void {
            addMessageLines += line;
        }
        
        
        private var diceCheckRegExp:RegExp = /^\s*ダイス合計[：:]\s*\d+\s*[\(\（]/;
        
        //ダイスロール結果を手入力で不正入力していないかのチェック
        private function isInvalidDiceRoll(chatMessage:String,
                                           isStand:Boolean,
                                           retryData:Object,
                                           isDiceRoll:Boolean):Boolean {
            //再送ならチェック無し
            if( retryData != null ) {
                return false;
            }
            
            //正当なダイスロール結果ならチェック無し
            if( isDiceRoll ) {
                return false;
            }
            
            var checkResult:Object = diceCheckRegExp.exec(chatMessage);
            if( checkResult == null ) {
                return false;
            }
            
            chatWindow.sendSystemMessage("のチャットメッセージに不正なダイスロール結果が検出されました。\n"
                                         + chatMessage + "");
            return true;
        }
        
        private var sendMessageIndex:int = 0;
        
        public function sendMessage(chatMessage:String,
                                    isStand:Boolean = true,
                                    retryData:Object = null,
                                    isDiceRoll:Boolean = false):void {
            var limitResult:Object = /^limit:(\d+)/.exec(chatMessage);//PLEASE_DELETE
            if( limitResult != null ) {//PLEASE_DELETE
                Log.logging("limitResult[0]", limitResult[0]);//PLEASE_DELETE
                thisObj.loadingLimitSeconds = parseInt(limitResult[1]);//PLEASE_DELETE
            } //PLEASE_DELETE
            
            if( isInvalidDiceRoll(chatMessage, isStand, retryData, isDiceRoll) ) {
                return;
            }
            
            var messageIndex:int = sendMessageIndex++;
            
            try{
                var name:String = chatWindow.chatCharacterName.text;
                
                if( retryData != null ) {
                    name = retryData.senderName;
                }
                
                if( isStand ) {
                    if( chatWindow.standingGraphicsStates.selectedItem != null ) {
                        var state:String = chatWindow.standingGraphicsStates.selectedItem.data;
                        name += "\t" + state;
                    }
                }
                
                var sendto:String = null;
                if( chatWindow.sendto.selectedItem ) {
                    sendto = chatWindow.sendto.selectedItem.data;
                }
                
                var time:Number = 0; //(new Date().getTime() / 1000);
                var isReplayChatMessage:Boolean = false;
                addMessageToChatLog(name, chatMessage, chatWindow.getChatFontColor(), time, localMessageUniqueId,
                                    isReplayChatMessage, sendto, messageIndex);
                
                var guiInputSender:GuiInputSender = DodontoF_Main.getInstance().getGuiInputSender();
                guiInputSender.sendChatMessage(name, chatMessage, chatWindow.getChatFontColor(), messageIndex, sendto);
            } catch(error:Error) {
                this.status = error.message;
            }
        }
        
        
        public function addLocalMessage(message:String):void {
            addMessageToChatLog(systemMessageSenderName, message, systemMessageColor, 0, "dummy");
        }
        
        public function sendSystemMessage(messageBase:String, isPrintName:Boolean = true):void {
            var isRoomResult:Boolean = true;
            var message:String = "";
            
            if( isPrintName ) {
                message += "「" + chatWindow.chatCharacterName.text + "」";
            }
            message += messageBase;
            
            var messageIndexTmp:int = -1;
            
            var guiInputSender:GuiInputSender = DodontoF_Main.getInstance().getGuiInputSender();
            guiInputSender.sendChatMessage(systemMessageSenderName, message, systemMessageColor, messageIndexTmp);
        }
        
        public function addMessageToChatLog(senderName:String,
                                            chatMessage:String,
                                            color:String,
                                            time:Number,
                                            chatSenderUniqueId:String,
                                            isReplayChatMessage:Boolean = false,
                                            sendto:String = null,
                                            messageIndex:int = -1):void {
            addMessageToChatLogParts(senderName,
                                     chatMessage,
                                     color,
                                     time,
                                     chatSenderUniqueId,
                                     messageIndex,
                                     sendto,
                                     isReplayChatMessage);
            
            printAddedMessageToChatMessageLog();
        }
        
        private function getOwnUniqueId():String {
            return DodontoF_Main.getInstance().getGuiInputSender().getSender().getUniqueId();
        }
        
        private function isPrintableMessageOnSecretMessage(chatSenderUniqueId:String, sendto:String):Boolean {
            
            if( chatSenderUniqueId == localMessageUniqueId ) {
                //"送信者が自分なら表示可能"
                return true;
            }
            
            if( sendto == null ) {
                //秘話指定無しなので表示可能
                return true;
            }
            
            if( sendto == getOwnUniqueId() ) {
                //秘話指定先が自分なら表示可能
                return true;
            }
            
            return false;
        }
        
        public function addMessageToChatLogParts(senderName:String,
                                                 chatMessage:String,
                                                 color:String,
                                                 time:Number,
                                                 chatSenderUniqueId:String,
                                                 messageIndex:int = -1,
                                                 sendto:String = null,
                                                 isReplayChatMessage:Boolean = false
                                                 ):void {
            
            Log.logging("addMessageToChatLog called");
            
            //自分の発言が戻ってきた場合は表示無し
            if( (senderName != systemMessageSenderName) && (senderName != "") ) {
                if( getOwnUniqueId() == chatSenderUniqueId ) {
                    recieveOwnMessage(messageIndex);
                    return;
                }
            }
            
            if( DodontoF_Main.getInstance().isReplayMode() ) {
                if( ! isReplayChatMessage ) {
                    var messageData:Object = [senderName, chatMessage, color, time, chatSenderUniqueId];
                    DodontoF_Main.getInstance().addRestChatMessageForReplay(messageData);
                    return;
                }
            }
            
            if( ! isPrintableMessageOnSecretMessage(chatSenderUniqueId, sendto) ) {
                //指定外のユーザーのためログ表示無し
                return;
            }
            
            
            var effectResult:Object = checkEffect(chatMessage, senderName);
            chatMessage = effectResult.chatMessage;
            senderName = effectResult.senderName;
            if( chatMessage == null ) {
                return;
            }
            
            senderName = escapeHtml(senderName);
            chatMessage = escapeHtml(chatMessage);
            var separator:String = "：";
            
            if( isReplayChatMessage ) {
                separator = "：<br>";
            }
            
            var messageLine:String = 
                "<font color='#" + color + "'>"
                + "<b>"
                + senderName
                + getSendtoName(sendto)
                + "</b>"
                + separator
                + chatMessage + "</font>";
            
            chatWindow.pushAllChatLogList( messageLine );
            
            if( addMessageLines != "" ) {
                addMessageLines += "<br>";
            }
            addMessageLines += "<font size='" + chatWindow.getChatFontSize() + "'>" + messageLine + "</font>";
            
            //printUpdateDateString(time);
            chatWindow.playAddMessageSound(senderName);
            chatWindow.setUserNames(senderName);
        
            Log.logging("addMessageToChatLog end");
        }
        
        private function getSendtoName(sendto:String):String {
            if( sendto == null ) {
                return "";
            }
            
            return " -> " + DodontoF_Main.getInstance().getDodontoF().getUserNameByUniqueId(sendto);
        }
        
        public static function escapeHtml(htmlString:String):String {
            //htmlString = htmlString.replace(/\"/g, "&quot;"); //"
            htmlString = htmlString.replace(/&/g, "&amp;");
            htmlString = htmlString.replace(/</g, "&lt;");
            htmlString = htmlString.replace(/>/g, "&gt;");
            return htmlString;
        }
        
        public function vote():void {
            voter.execute();
        }
        
        private function checkEffect(chatMessage:String, senderName:String):Object {
            var result:Object = {
                "chatMessage" : chatMessage, 
                "senderName"  : senderName
            };
            
            var guiInputSender:GuiInputSender = DodontoF_Main.getInstance().getGuiInputSender();
            var effectable:Boolean = (! guiInputSender.getSender().getReciever().isFirstRefresh());
            
            
            if( effectable && chatWindow.isAlarm( chatMessage ) ) {
                chatWindow.playSound(chatMessage);
            }
            
            var standingGraphicFilterImages:Array = new Array();
            result.chatMessage = voter.received( chatMessage, effectable, standingGraphicFilterImages );
            
            
            var printResult:Object = chatWindow.printStandingGraphics(senderName, chatMessage, effectable,
                                                                      standingGraphicFilterImages);
            
            if( printResult != null ) {
                result.senderName = printResult.senderName;
            }
            
            
            for(var i:int = 0 ; i < cutInList.length ; i++) {
                var cutIn:CutInBase = cutInList[i];
                
                //checkDelayCutIn(chatMessage, cutIn, effectable);
                
                var matchResult:Object = cutIn.matchCutIn(chatMessage);
                
                addArrayToArray(matchResult.delays, cutInDelayInfoList);
                
                if( matchResult.resultData == null ) {
                    continue;
                }
                
                cutIn.setSoundOn( chatWindow.isSoundOnMode() );
                cutIn.setEffectable(effectable);
                result.chatMessage = cutIn.effect( matchResult.resultData.chatMessage );
            }
            
            return result;
        }
        
        private function addArrayToArray(from:Array, to:Array):void {
            for(var i:int = 0 ; i < from.length ; i++) {
                to.push(from[i]);
            }
        }
        
        private function checkDelayCutIn(chatMessage:String, cutIn:CutInBase, effectable:Boolean):void {
            var hitIndexs:Array = new Array();
            for(var i:int = 0 ; i < cutInDelayInfoList.length ; i++) {
                var matchResult:Object = cutInDelayInfoList[i];
                
                if( ! cutIn.isTargetMarker( matchResult.chatMessage ) ) {
                    continue;
                }
                
                var cutInInfo:Object = matchResult.cutInInfo;
                
                if( cutInInfo.trigerTail == null ) {
                    continue;
                }
                
                var cancelTail:String = "失敗";
                if( cutInInfo.trigerTail == "失敗" ) {
                    cancelTail = "成功";
                }
                
                if( CutInBase.isTail(chatMessage, cancelTail) ) {
                    hitIndexs.push(i);
                    continue;
                }
                
                if( ! CutInBase.isTail(chatMessage, cutInInfo.trigerTail) ) {
                    continue;
                }
                
                hitIndexs.push(i);
                
                cutIn.setSoundOn( chatWindow.isSoundOnMode() );
                cutIn.setEffectable(effectable);
                //result.chatMessage = 
                cutIn.effect( matchResult.chatMessage );
            }
            
            deleteTargetIndex(cutInDelayInfoList, hitIndexs);
        }
        
        private function deleteTargetIndex(array:Array, indexs:Array):void {
            while( indexs.length > 0 ) {
                var index:int = indexs.pop();
                array.splice(index, 1);
            }
        }
        
        private function getRetryBlock(message:String):HBox {
            var box:HBox = new HBox();
            box.percentWidth = 100;
            
            var retryData:Object = getRetryData(message);
            
            if( retryData == null ) {
                return null;
            }
            
            var retryButton:Button = new Button();
            retryButton.label = "再送";
            retryButton.addEventListener(MouseEvent.CLICK, function(event:Event):void {
                    var isStand:Boolean = true;
                    thisObj.sendMessage(retryData.message, isStand, retryData);
                    
                    try {
                        thisObj.chatWindow.chatMessageLogBox.removeChild( box );
                    } catch(e:Error) {
                    }
                });
            
            var cancelButton:Button = new Button();
            cancelButton.label = "キャンセル";
            cancelButton.addEventListener(MouseEvent.CLICK, function(event:Event):void {
                    try {
                        thisObj.chatWindow.chatMessageLogBox.removeChild( box );
                    } catch(e:Error) {
                    }
                });
            
            var text:TextArea = getFaildMessageTextArea(message);
            box.addChild(retryButton);
            box.addChild(cancelButton);
            box.addChild(text);
            
            return box;
        }
        
        private function getFaildMessageTextArea(message:String):TextArea {
            var text:TextArea = new TextArea();
            text.htmlText = message;
            
            text.percentWidth = 100;
            text.height = 25;
            text.editable = false;
            
            text.setStyle("paddingTop", 0);
            text.setStyle("paddingBottom", 0);
            text.setStyle("horizontalGap", 0);
            text.setStyle("verticalGap", 0);
            text.setStyle("leading", 0);
            
            return text;
        }
        
        private function sendLoadingFailed(messageIndex:int):void {
            Log.logging("sendLoadingFailed messageIndex", messageIndex);
            
            var obj:Object = sendedChatMessageData[messageIndex];
            Log.logging("sendLoadingFailed obj", obj);
            if( obj == null ) {
                Log.logging("sendLoadingFailed obj is null, so send if scuessed.");
                return;
            }
            
            var message:String = obj.message;
            Log.logging("sendLoadingFailed message", message);
            if( message == null ) {
                return;
            }
            
            var htmlText:String = "送信失敗：" + message;
            
            var retryBlock:HBox = getRetryBlock(htmlText);
            if( retryBlock == null ) {
                return;
            }
            
            obj.retryBlock = retryBlock;
            
            chatWindow.chatMessageLogBox.addChild( retryBlock );
            Log.logging("sendLoadingFailed text added");
        }
        
        private function getRetryData(str:String):Object {
            var regExp:RegExp = /<b>(.*?)<\/b>：(.*?)</i;
            var regResult:Object = regExp.exec(str);
            if( regResult == null ) {
                return null;
            }
            
            var senderName:String = regResult[1];
            var message:String = regResult[2];
            
            var retryData:Object = {
                senderName: senderName,
                message: message
            }
            
            return retryData;
        }
        
        private function recieveOwnMessage(messageIndex:int):void {
            Log.logging("recieveOwnMessage called");
            
            var obj:Object = sendedChatMessageData[messageIndex];
            sendedChatMessageData[messageIndex] = null;
            
            if( obj == null ) {
                return;
            }
            
            if( obj.retryBlock == null ) {
                return;
            }
            
            try {
                thisObj.chatWindow.chatMessageLogBox.removeChild( obj.retryBlock );
            } catch(e:Error) {
            }
        }
        
        private function getImageId():String {
            return "loadingImage";
        }
        
        private function getChatMarkerImage(imageUrl:String):String {
            var imageSize:int = chatWindow.getChatFontSize();
            imageUrl = Config.getInstance().getUrlString(imageUrl); //PLEASE_DELETE
            
            return '<img src="' + imageUrl + '" '
                + 'vspace="0" hspace="2" '
                + 'id="' + getImageId() + '" '
                + 'width="' + imageSize + '" '
                + 'height="' + imageSize + '" '
                + '>';
        }
        
        public function printAddedMessageToChatMessageLog():void {
            chatWindow.addChatLogText(addMessageLines);
            clearAddMessageLines();
            chatWindow.scrollChatMessageLogIfPositionIsLast(false);
        }
        
    }
}
