//--*-coding:utf-8-*--
package {
    import mx.utils.StringUtil;
    import flash.net.FileReference;
    
	public class GuiInputSender {
        
        private var sender:SharedDataSender = new SharedDataSender();
        private var feetsLimit:int = 100;
        private var characterCreationLimit:int = 10;
        private var maskCreationLimit:int = 100;
        private var playRoomMaxNumber:int = 100;
        
        public function GuiInputSender() {
        }
        
        public function setSender(sender_:SharedDataSender):void {
            sender = sender_;
        }

        public function getSender():SharedDataSender {
            return sender;
        }
        
        public function save(resultFunction:Function):void {
            sender.save(resultFunction);
        }
        
        public function load(params:Object):void {
            sender.load(params);
        }
        
        public function checkRoomNumber(roomNumberString:String):int {
            if( sender.isSampleMode() ) {
                return parseInt(roomNumberString);
            }
            
            var roomNumber:int = parseInt(roomNumberString);
            if( (roomNumber < 0) || (roomNumber > playRoomMaxNumber) ) {
                throw new Error("ルームNo.は0〜" + playRoomMaxNumber + "の値を設定してください。");
            }
            return roomNumber;
        }
        
        public function checkRoomStatus(roomNumberString:String, resultFunction:Function):void {
            var roomNumber:int = checkRoomNumber(roomNumberString);
            sender.checkRoomStatus(roomNumber, resultFunction);
        }
        
        public function login(roomNumber:int, password:String, resultFunction:Function):void {
            sender.login(roomNumber, password, resultFunction);
        }
        
        public function getRoomNumberName():String {
            var number:int = sender.getRoomNumber();
            if( number > playRoomMaxNumber ) {
                return "お試しルーム";
            }
            
            return "ルーム" + number;
        }
        
        public function uploadImageFile(fileReferenceForImageUpload:FileReference,
                                        params:Object):void {
            sender.uploadImageFile(fileReferenceForImageUpload, params);
        }
        
        public function changeMap(mapImageUrl:String,
                                  mapWidth:int,
                                  mapHeight:int,
                                  gridColor:uint,
                                  mapMarks:Array):void {
            if( ( mapHeight < 1 ) || ( mapHeight > 100 ) ||
                ( mapWidth < 1 ) || ( mapWidth > 100 ) ) {
                throw new Error("縦・横マス数の入力値が不正です。1〜100の整数を入力してください。");
            }
            
            sender.changeMap(mapImageUrl, mapWidth, mapHeight, gridColor, mapMarks);
        }
        
        public function deleteImage(imageUrlList:Array,
                                    resultFunction:Function):void {
            sender.deleteImage(imageUrlList, resultFunction);
        }
        
        public function getAndCheckAddCharacterParams(characterName:String, imageUrl:String,
                                                      size:int, isHide:Boolean,
                                                      initiative:Number, info:String, direction:int):Object {
            Log.logging("addCharacter start");
            Log.logging("characterName", characterName);
            Log.logging("imageUrl", imageUrl);
            Log.logging("size", size);
            Log.logging("initiative", initiative);
            Log.logging("info", info);
            
            if( characterName.length == 0 ) {
                throw new Error("名前を入力してください。");
            }
            
            if( imageUrl.length == 0 ) {
                throw new Error("イメージ画像のURLを入力してください。");
            }
            
            if( ( size < 1 ) || ( size > 10 ) ) {
                throw new Error("サイズの入力値が不正です。1〜10の整数を入力してください。");
            }
            
            if( ( initiative < 0 ) ) {
                throw new Error("イニシアティブの入力値が不正です。0以上の数値を入力してください。");
            }
            
            Log.logging("createLoop");
            
            Log.logging("sender.addCharacter called");
            var createName:String = characterName;
            
            var characterParams:Object = new Object;
            characterParams.name = characterName;
            characterParams.imageUrl = imageUrl;
            characterParams.size = size;
            characterParams.isHide = isHide;
            characterParams.initiative = initiative;
            characterParams.info = info;
            characterParams.direction = direction;
            
            return characterParams;
        }
        
        public function addCharacter(name:String, imageUri:String,
                                     size:int, isHide:Boolean,
                                     initiative:int, info:String, direction:int,
                                     characterPositionX:int, characterPositionY:int):void {
            
            var characterJsonData:Object =
                Character.getJsonData(name, imageUri,
                                      size, isHide,
                                      initiative, info, direction, 0,
                                      characterPositionX, characterPositionY);
            sender.addCharacter(characterJsonData);
            
            Log.logging("addCharacter end");
        }
        
        public function addMapMask(width:int,
                                   height:int,
                                   name:String,
                                   color:int,
                                   alpha:Number,
                                   createPositionX:int, createPositionY:int):void {
            Log.logging("addMapMask start");
            Log.logging("width", width);
            Log.logging("height", height);
            Log.logging("color", color);
            Log.logging("GuiInputSender createPositionX", createPositionX);
            Log.logging("GuiInputSender createPositionY", createPositionY);
            
            if( width < 0 ) {
                throw new Error("幅が不正な値です。");
            }
            
            if( height < 0 ) {
                throw new Error("高さが不正な値です。");
            }
            
            if( (createPositionX < 0) || (createPositionY < 0) ) {
                throw new Error("設置X,Y座標が0未満の値になっています。");
            }
            
            var characterJsonData:Object =
                MapMask.getJsonData(width, height, name, color, alpha, createPositionX, createPositionY);
            sender.addCharacter(characterJsonData);
            
            Log.logging("addCharacter end");
        }
        
        public function addMapMarker(message:String, color:uint,
                                     isPaint:Boolean,
                                     width:int, height:int,
                                     characterPositionX:int, characterPositionY:int):void {
            if( message.length == 0 ) {
                throw new Error("メッセージを入力してください。");
            }
            
            if( (characterPositionX < 0) || (characterPositionY < 0) ) {
                throw new Error("設置X,Y座標が0未満の値になっています。");
            }
            
            var characterJsonData:Object =
                MapMarker.getJsonData(message, color,
                                      isPaint, 
                                      width, height, 
                                      characterPositionX, characterPositionY);
            
            sender.addCharacter(characterJsonData, "message");
            
            Log.logging("addMagicRange end");
        }
        
        public function addMagicRange(name:String, feets:int, type:String, color:String,
                                      timeRange:int,
                                      createRound:Number,
                                      initiative:Number,
                                      info:String,
                                      characterPositionX:int, characterPositionY:int):void {
            Log.logging("GuiInputSender addMagicRange characterPositionX", characterPositionX);
            Log.logging("GuiInputSender addMagicRange characterPositionY", characterPositionY);
            Log.logging("timeRange", timeRange);
            Log.logging("createRound", createRound);
            Log.logging("initiative", initiative);
            Log.logging("info", info);
            
            if( name.length == 0 ) {
                throw new Error("名前を入力してください。");
            }
            
            if( ( feets < 1 ) || ( feets > feetsLimit ) || ((feets % 5) != 0) ) {
                throw new Error("半径の入力値が不正です。1〜" + feetsLimit + "の整数で5の倍数を入力してください。");
            }
            
            if( type.length == 0 ) {
                throw new Error("魔法範囲種別を選択してください。");
            }
            
            if( color.length == 0 ) {
                throw new Error("色を選択してください");
            }
            
            if( (characterPositionX < 0) || (characterPositionY < 0) ) {
                throw new Error("設置X,Y座標が0未満の値になっています。");
            }
            
            if( ( initiative < 0 ) ) {
                throw new Error("イニシアティブの入力値が不正です。0以上の数値を入力してください。");
            }
            
            var characterJsonData:Object =
                MagicRange.getJsonData(name, feets, type, color,
                                       timeRange,
                                       createRound,
                                       initiative,
                                       info,
                                       characterPositionX, characterPositionY);
            sender.addCharacter(characterJsonData);

            Log.logging("addMagicRange end");
        }

        public function addMagicRangeDD4th(name:String,
                                           feets:int,
                                           type:String,
                                           color:String,
                                           timeRange:int,
                                           createRound:Number,
                                           initiative:Number,
                                           info:String,
                                           characterPositionX:int,
                                           characterPositionY:int):void {
            Log.logging("GuiInputSender addMagicRange characterPositionX", characterPositionX);
            Log.logging("GuiInputSender addMagicRange characterPositionY", characterPositionY);
            Log.logging("timeRange", timeRange);
            Log.logging("createRound", createRound);
            Log.logging("initiative", initiative);
            Log.logging("info", info);
            
            if( name.length == 0 ) {
                throw new Error("名前を入力してください。");
            }
            
            if( ( feets < 1 ) || ( feets > feetsLimit ) ) {
                throw new Error("半径の入力値が不正です。1〜" + feetsLimit + "の整数を入力してください。");
            }
            
            if( type.length == 0 ) {
                throw new Error("魔法範囲種別を選択してください。");
            }
            
            if( color.length == 0 ) {
                throw new Error("色を選択してください");
            }
            
            if( (characterPositionX < 0) || (characterPositionY < 0) ) {
                throw new Error("設置X,Y座標が0未満の値になっています。");
            }
            
            if( ( initiative < 0 ) ) {
                throw new Error("イニシアティブの入力値が不正です。0以上の数値を入力してください。");
            }
            
            var characterJsonData:Object =
                MagicRangeDD4th.getJsonData(name, feets, type, color,
                                            timeRange,
                                            createRound,
                                            initiative,
                                            info,
                                            characterPositionX, characterPositionY);
            Log.logging("addMagicRangeDD4th sender.addCharacter begin");
            sender.addCharacter(characterJsonData);
            Log.logging("addMagicRangeDD4th sender.addCharacter end");
            
            Log.logging("addMagicRange end");
        }

        public function addMagicTimer(name:String,
                                      timeRange:int,
                                      createRound:Number,
                                      initiative:Number,
                                      info:String):void {
            Log.logging("timeRange", timeRange);
            Log.logging("createRound", createRound);
            Log.logging("initiative", initiative);
            Log.logging("info", info);
            
            if( name.length == 0 ) {
                throw new Error("名前を入力してください。");
            }
            
            if( ( initiative < 0 ) ) {
                throw new Error("イニシアティブの入力値が不正です。0以上の数値を入力してください。");
            }
            
            var characterJsonData:Object =
                MagicTimer.getJsonData(name,
                                       timeRange,
                                       createRound,
                                       initiative,
                                       info);
            sender.addCharacter(characterJsonData);
            
            Log.logging("addMagicTimer end");
        }
        
        public function changeCharacter(character:Character,
                                        name:String, imageUri:String,
                                        size:int, isHide:Boolean,
                                        initiative:Number, info:String,
                                        counters:Object):void {
            Log.logging("changeCharacter start");
            Log.logging("name", name);
            Log.logging("imageUri", imageUri);
            Log.logging("size", size);
            Log.logging("initiative", initiative);
            Log.logging("info", info);
            
            if( name.length == 0 ) {
                throw new Error("名前を入力してください。");
            }
            
            if( imageUri.length == 0 ) {
                throw new Error("イメージ画像のURLを入力してください。");
            }
            
            if( ( size < 1 ) || ( size > 10 ) ) {
                throw new Error("サイズの入力値が不正です。1〜10の整数を入力してください。");
            }
            
            if( ( initiative < 0 ) ) {
                throw new Error("イニシアティブの入力値が不正です。0以上の数値を入力してください。");
            }
            /*
            var characterJsonData:Object =
                character.getJsonDataForChange(name,
                                               imageUri,
                                               size,
                                               initiative,
                                               info);
            */
            character.setName(name);
            character.setImageUrl(imageUri);
            character.setSize(size);
            character.setHide(isHide);
            character.setInitiative(initiative);
            character.setInfo(info);
            //character.setCounters(counters);
            
            sender.changeCharacter( character.getJsonData() );
            
            Log.logging("changeCharacter end");
        }
        
        public function changeMagicRange(magicRange:MagicRange,
                                         name:String,
                                         feets:int,
                                         rangeType:String,
                                         color:String,
                                         info:String,
                                         timeRange:int):void {
            Log.logging("changeMagicRange start");
            Log.logging("name", name);
            Log.logging("rangeType", rangeType);
            Log.logging("feet", feets);
            Log.logging("info", info);
            Log.logging("timeRange", timeRange);
            
            if( name.length == 0 ) {
                throw new Error("名前を入力してください。");
            }
            
            if( ( feets < 1 ) || ( feets > feetsLimit ) || ((feets % 5) != 0) ) {
                throw new Error("半径の入力値が不正です。1〜" + feetsLimit + "の整数で5の倍数を入力してください。");
            }
            
            if( rangeType.length == 0 ) {
                throw new Error("範囲タイプを選択してください。");
            }
            
            if( color.length == 0 ) {
                throw new Error("色を選択してください。");
            }
            
            if( timeRange < 1 ) {
                throw new Error("持続時間の値範囲が不正です。1以上の値を入力してください。");
            }
            
            magicRange.setName(name);
            magicRange.setFeets(feets);
            magicRange.setRangeType(rangeType);
            magicRange.setColor( parseInt(color) );
            magicRange.setInfo(info);
            magicRange.setTimeRange(timeRange);
            
            sender.changeCharacter( magicRange.getJsonData() );
            
            Log.logging("changeMagicRange end");
        }
        
        public function changeMagicRangeDD4th(magicRange:MagicRange,
                                         name:String,
                                         feets:int,
                                         rangeType:String,
                                         color:String,
                                         info:String,
                                         timeRange:int):void {
            Log.logging("changeMagicRangeDD4th start");
            Log.logging("name", name);
            Log.logging("rangeType", rangeType);
            Log.logging("feet", feets);
            Log.logging("info", info);
            Log.logging("timeRange", timeRange);
            
            if( name.length == 0 ) {
                throw new Error("名前を入力してください。");
            }
            
            if( ( feets < 1 ) || ( feets > feetsLimit ) ) {
                throw new Error("半径の入力値が不正です。1〜" + feetsLimit + "の整数で5の倍数を入力してください。");
            }
            
            if( rangeType.length == 0 ) {
                throw new Error("範囲タイプを選択してください。");
            }
            
            if( color.length == 0 ) {
                throw new Error("色を選択してください。");
            }
            
            if( timeRange < 1 ) {
                throw new Error("持続時間の値範囲が不正です。1以上の値を入力してください。");
            }
            
            magicRange.setName(name);
            magicRange.setFeets(feets);
            magicRange.setRangeType(rangeType);
            magicRange.setColor( parseInt(color) );
            magicRange.setInfo(info);
            magicRange.setTimeRange(timeRange);
            
            sender.changeCharacter( magicRange.getJsonData() );
            
            Log.logging("changeMagicRangeDD4th end");
        }
        
        public function changeMagicTimer(magicTimer:MagicTimer,
                                         name:String,
                                         timeRange:int,
                                         createRound:Number,
                                         initiative:Number,
                                         info:String):void {
            Log.logging("name", name);
            Log.logging("timeRange", timeRange);
            Log.logging("createRound", createRound);
            Log.logging("initiative", initiative);
            Log.logging("info", info);
            
            if( name.length == 0 ) {
                throw new Error("名前を入力してください。");
            }
            
            if( ( initiative < 0 ) ) {
                throw new Error("イニシアティブの入力値が不正です。0以上の数値を入力してください。");
            }
            
            magicTimer.setName(name);
            magicTimer.setInfo(info);
            magicTimer.setInitiative(initiative);
            magicTimer.setTimeRange(timeRange);
            magicTimer.setCreateRound(createRound);
            
            sender.changeCharacter( magicTimer.getJsonData() );
            
            Log.logging("addMagicTimer end");
        }
        
        public function resurrectCharacter(resurrectCharacterId:String):void {
            Log.logging("resurrectCharacterId", resurrectCharacterId);
            
            if( resurrectCharacterId.length == 0 ) {
                throw new Error("復活させたいキャラクターのＩＤを入力してください。");
            }
            
            sender.resurrectCharacter(resurrectCharacterId);
            Log.logging("resurrectCharacter end");
        }
        
        public function requestGraveyard(requestGraveyardResult:Function):void {
            sender.requestGraveyard(requestGraveyardResult);
        }
        
        public function requestLoginInfo(resultFunction:Function):void {
            sender.requestLoginInfo(resultFunction);
        }
        
        public function requestSaveChatLog(chatLogHtml:String,
                                           resultFunction:Function):void {
            sender.requestSaveChatLog(chatLogHtml, resultFunction);
        }
        
        public function getDiceBotText(chatMessage:String, randomSeed:int, gameType:String, callBack:Function):void {
            sender.getDiceBotText(chatMessage, randomSeed, gameType, callBack);
        }
        
        public function sendChatMessage(chatCharacterName:String, chatMessage:String, color:String, messageIndex:int, sendto:String = null):void {
            if( chatCharacterName.length == 0 ) {
                throw new Error("発言者を入力してください。");
            }
            if( chatMessage.length == 0 ) {
                //throw new Error("空メッセージは送信できません。");
                return;
            }
            
            sender.sendChatMessage(chatCharacterName, chatMessage, color, messageIndex, sendto);
        }
        
        public function requestSaveSessionLog(sessionLog:String,
                                              resultFunction:Function):void {
            sender.requestSaveSessionLog(sessionLog, resultFunction);
        }
        
        public function sendChatMessageMany(chatCharacterName:String, chatMessage:String, color:String):void {
            if( chatCharacterName.length == 0 ) {
                throw new Error("発言者を入力してください。");
            }
            if( chatMessage.length == 0 ) {
                //throw new Error("空メッセージは送信できません。");
                return;
            }
            
            sender.sendChatMessageMany(chatCharacterName, chatMessage, color);
        }
        
        public function createPlayRoom(playRoomName:String,
                                       playRoomPassword:String,
                                       playRoomIndex:int,
                                       resultFunction:Function):void {
            sender.createPlayRoom(playRoomName, playRoomPassword, playRoomIndex, resultFunction);
        }
        
        public function setSaveDataDirIndex(roomNumber:int):void {
            sender.setSaveDataDirIndex(roomNumber);
        }
        
        public function requestImageList(result:Function):void {
            sender.requestImageList(result)
        }
        
        public function uploadImageUrl(imageUrl:String, tagInfo:Object, resultFunction:Function):void {
            sender.uploadImageUrl(imageUrl, tagInfo, resultFunction);
        }
        
        public function removeCharacter(piece:Piece, isGotoGraveyard:Boolean = true):void {
            sender.removeCharacter(piece, isGotoGraveyard);
        }
        
        public function removePlayRoom(roomNumber:int, resultFunction:Function, ignoreLoginUser:Boolean):void {
            sender.removePlayRoom(roomNumber, resultFunction, ignoreLoginUser);
        }
        
        public function addEffect(params:Object):void {
            sender.addEffect(params);
        }
        
        public function changeEffect(params:Object):void {
            sender.changeEffect(params);
        }
        
        public function removeEffect(effectId:String):void {
            sender.removeEffect(effectId);
        }
    }
}
