//--*-coding:utf-8-*--

package {
    
    import mx.controls.Alert;
    import flash.events.ContextMenuEvent;
    import flash.ui.ContextMenu;
    import flash.ui.ContextMenuItem;
    import com.adobe.serialization.json.JSON;
    import flash.events.MouseEvent;
	import mx.core.UIComponent;
    
    public class PaintablePreviewLoader extends PreviewLoader {
        private static var eraseColor:int = PaintablePreviewLoaderSquare.getEraseColor();
        
        private var paintSquare:UIComponent = new UIComponent();
        private var thisObj:PaintablePreviewLoader;
        private var squaresList:Array = new Array();
        
        public function PaintablePreviewLoader() {
            super();
            this.addChild(paintSquare);
            this.thisObj = this;
        }
        
        private var getColorFunction:Function = null;
        private var setColorFunction:Function = null;
        private var isDrawing:Boolean = false;
        private var isErasing:Boolean = false;
        private var isSyringe:Boolean = false;
        
        public function setEraser(isErasing_:Boolean = true):void {
            isErasing = isErasing_;
        }
        
        public function setSyringe(isSyringe_:Boolean = true):void {
            isSyringe = isSyringe_;
        }
        
        private var syringeAfter:Function;
        
        public function setSyringeAfter(f_:Function):void {
            syringeAfter = f_;
        }
        
        public function getResult():Array {
            var result:Array = new Array(getHeightSize());
            
            for(var y:int = 0 ; y < result.length; y++) {
                result[y] = new Array( getWidthSize() );
                for(var x:int = 0 ; x < result[y].length ; x++) {
                    result[y][x] = squaresList[y][x].getColor();
                }
            }
            
            return result;
        }
        
        public function clearSquares():void {
            for(var y:int = 0 ; y < getHeightSize() ; y++) {
                var squares:Array = squaresList[y];
                if( squares == null ) {
                    continue;
                }
                
                for(var x:int = 0 ; x < getWidthSize() ; x++) {
                    var square:PaintablePreviewLoaderSquare = squares[x];
                    try {
                        this.paintSquare.removeChild(square);
                    } catch (e:Error) {
                    }
                }
            }
        }
        
        public function drawSquare(mapMarks:Array,
                                   getColorFunction_:Function,
                                   setColorFunction_:Function):void {
            getColorFunction = getColorFunction_;
            setColorFunction = setColorFunction_;
            
            isDrawing = true;
            var length:Number = getLength();
            
            squaresList = new Array( getHeightSize() );
            for(var y:int = 0 ; y < getHeightSize(); y++) {
                squaresList[y] = new Array( getWidthSize() );
                for(var x:int = 0 ; x < getWidthSize() ; x++) {
                    var square:PaintablePreviewLoaderSquare = new PaintablePreviewLoaderSquare();
                    squaresList[y][x] = square;
                    square.x = x * length;
                    square.y = y * length;
                    square.width = length;
                    square.height = length;
                    square.setPosition(x, y);
                    
                    var color:int = eraseColor;
                    if( mapMarks[y] != null ) {
                        if( mapMarks[y][x] != null ) {
                            color = mapMarks[y][x];
                        }
                    }
                    this.drawSquareOne(square, color);
                    
                    this.paintSquare.addChild(square);
                    
                    square.addEventListener(MouseEvent.MOUSE_DOWN, 
                                            function(event:MouseEvent):void {
                                                thisObj.isDrawing = true;
                                                Log.logging("mouse DWON======");
                                                thisObj.mouseOverEventHandler(event);
                                            });
                    square.addEventListener(MouseEvent.MOUSE_UP, 
                                            function(event:MouseEvent):void {
                                                thisObj.isDrawing = false;
                                                Log.logging("mouse UP========");
                                            });
                    square.addEventListener(MouseEvent.MOUSE_OVER,
                                          thisObj.mouseOverEventHandler);
                    initContextMenu(square);
                }
            }
            isDrawing = false;
        }
        
        private function initContextMenu(square:PaintablePreviewLoaderSquare):void {
            var menu:ContextMenu = new ContextMenu();
            menu.hideBuiltInItems();
            
            MovablePiece.addMenuItem(menu, "色取得（スポイト）", thisObj.getFunctionForSyringeFromMenu(square));
            
            square.contextMenu = menu;
        }
        
        private function getFunctionForSyringeFromMenu(view:PaintablePreviewLoaderSquare):Function {
            return function(event:ContextMenuEvent):void {
                pickupColorForce(view);
            }
        }
        
        private function mouseOverEventHandler(event:MouseEvent):void {
            try {
                var view:PaintablePreviewLoaderSquare
                    = event.currentTarget as PaintablePreviewLoaderSquare;
                /*
                var self:PaintablePreviewLoader
                    = event.currentTarget as PaintablePreviewLoader;
                */
                /*
                var x:int = event.localX / getLength();
                var y:int = event.localY / getLength();
                var view:PaintablePreviewLoaderSquare = this.squaresList[y][x];
                */
                
                if( isSyringe ) {
                    pickupColor(view);
                    if( syringeAfter != null ) {
                        syringeAfter.call();
                    }
                    return;
                }
                
                var color:int = eraseColor;
                if( ! isErasing ) {
                    color = getColorFunction.call();
                }
                drawSquareOne(view, color);
            } catch(error:Error) {
                Log.loggingException("PreviewLoader.mouseOverEventHandler()", error);
            }
        }
        
        private function pickupColor(view:PaintablePreviewLoaderSquare, isForce:Boolean = false):void {
            if( ! isDrawing ) {
                    return;
            }
            
            pickupColorForce(view);
        }
        private function pickupColorForce(view:PaintablePreviewLoaderSquare):void {
            Log.logging("view.getColor()", view.getColor());
            setColorFunction.call(null, view.getColor());
            isDrawing = false;
            Log.logging("pickupColor!!!!!!!!!");
        }
        
        private function drawSquareOne(view:PaintablePreviewLoaderSquare, color:int):void {
            Log.logging("isDrawing", isDrawing);
            if( ! isDrawing ) {
                return;
            }
            Log.logging("color", color);
            
            var length:Number = getLength();
            
            var x:int = view.x / length;
            var y:int = view.y / length;
            
            view.setColor(color);
            view.drawSquare();
        }
    }
}
