//--*-coding:utf-8-*--

package {
    
    import com.adobe.serialization.json.JSON;
    import flash.events.Event;
    import flash.events.DataEvent;
    import flash.net.URLLoader;
    import flash.net.FileReference;
    import flash.net.URLRequest;
    import flash.utils.ByteArray;
    import mx.collections.ArrayCollection;
    
    
    public class SharedDataReceiver {
        private var sender:SharedDataSender = null;
        private var chatMessageDataLastWrittenTime:Number = 0;
        private var roundTimer:RoundTimer;
        private var isFirstRefreshFlag:Boolean = true;
        private var fileReferenceForDownload:FileReference = new FileReference();
        private var map:Map = null;
        private var history:Array = new Array();
        private var isHistoryOn:Boolean = false;
        
        private static var dodontoF:DodontoF;
        
        public static function setDodontoF(dodontoF_:DodontoF):void {
            dodontoF = dodontoF_;
        }
        
        public function setSender(sender_:SharedDataSender):void {
            sender = sender_;
        }
        public function setMap(map_:Map):void {
            map = map_;
        }
        
        public function setRoundTimer(roundTimer_:RoundTimer):void {
            roundTimer = roundTimer_;
        }
        
        public function analyzeRefreshResponse(event:Event):void {
            Log.loggingTuning("==>Begin analyzeRefreshResponse");
            
            if( sender.isStopRefresh() ) {
                Log.logging("analyzeRefreshResponse(event:Event): refresh is stoped, so no analyze.");
                return;
            }
            
            var isValidResponse:Boolean = false;
            var jsonData:Object = null;
            try {
                Log.logging("analyzeRefreshResponse(event:Event):");
                var result:Object = analyzeRefreshResponseCatched(event);
                isValidResponse = result.isValidResponse;
                jsonData = result.jsonData;
            } catch(e:Error) {
                //FireFox 3.5系でこの応答エラーが頻発する問題があったためエラーログをコメントアウト
                //Log.loggingException("SharedDataReceiver.analyzeRefreshResponse()", e);
            }
            
            Log.loggingTuning("==>End analyzeRefreshResponse");
            
            if( ! isValidResponse ) {
                return;
            }
            
            if( isHistoryOn ) {
                history.push(jsonData);
            }
            
            if( sender.clearRetry() ) {
                Log.loggingErrorOnStatus("サーバとの再接続に成功しました。");
            }
            
            refreshNext();
        }
        
        public function clearChatLastWrittenTime():void {
            chatMessageDataLastWrittenTime = 0;
        }
        
        public function initForFirstRefresh():void {
            isFirstRefreshFlag = false;
            DodontoF_Main.getInstance().initForFirstRefresh();
        }
        
        public function isFirstRefresh():Boolean {
            return isFirstRefreshFlag;
        }
        
        protected function refreshNext():void {
            refresh();
        }
        
        private function refresh():void {
            sender.refresh();
        }
        
        private function analyzeRefreshResponseCatched(event:Event):Object {
            var jsonData:Object = getJsonDataFromResultEvent(event);
            var isValidResponse:Boolean = this.analyzeRefreshResponseCatchedCallByJsonData(jsonData);
            
            return {"isValidResponse" : isValidResponse,
                    "jsonData" : jsonData};
        }
        
        public static function getJsonDataFromResultEvent(event:Event):Object {
            var loader:URLLoader = URLLoader(event.target);
            var jsonString:String = loader.data;
            jsonString = getJsonStringFromEventResultJsonString(jsonString);
            
            var jsonData:Object = getJsonDataFromString(jsonString);
            return jsonData;
        }
        
        public static function getJsonDataFromDataEvent(dataEvent:DataEvent):Object {
            var jsonString:String = dataEvent.data;
            jsonString = getJsonStringFromEventResultJsonString(jsonString);
            
            var jsonData:Object = getJsonDataFromString(jsonString);
            return jsonData;
        }
        
        static private var regJsonStringResult:RegExp = /#D@EM>#(.*)#<D@EM#/ms;
        
        private static function getJsonStringFromEventResultJsonString(jsonStringOriginal:String):String {
            var textResult:Object = regJsonStringResult.exec(jsonStringOriginal);
            var jsonString:String = textResult[1];
            
            if( jsonString == null ) {
                jsonString = jsonStringOriginal;
            }
            
            return jsonString;
        }
        
        public static function getJsonDataFromString(jsonString:String):Object {
            Log.logging("jsonString : " + jsonString);
            
            var jsonData:Object = null;
            try {
                if( jsonString == null ) {
                    jsonData = new Object();
                } else {
                    jsonData = JSON.decode(jsonString);
                }
            } catch( e:Error ) {
                //Log.loggingException("SharedDataReceiver.getJsonDataFromString()", e);
            }
            
            return jsonData;
        }
        
        public function analyzeRefreshResponseCatchedCallByJsonData(jsonData:Object):Boolean {
        
            Log.logging("analyzeRefreshResponseCatchedCallByJsonData jsonData", jsonData);
            
            if( jsonData == null ) {
                Log.logging("jsonData is null");
                return false;
            }
            
            if( ! DodontoF_Main.getInstance().isReplayMode() ) {
                if( jsonData.refreshIndex == null ) {
                    Log.loggingError("refreshIndex is null. this is invalid data.");
                    return false;
                }
                
                Log.logging("jsonData.refreshIndex", jsonData.refreshIndex);
                if( ! sender.clearLastRefreshIndex(jsonData.refreshIndex) ) {
                    Log.loggingTuning("jsonData.refreshIndex is not lastRefreshIndex");
                    return false;
                }
                
                if( jsonData.loginUserInfo == null ) {
                    Log.loggingError("loginUserInfo is null. this is invalid data.");
                    return false;
                }
                dodontoF.setLoginUserInfo(jsonData.loginUserInfo);
            }
            
            
            if( jsonData.chatMessageDataLog &&
                sender.checkLastUpdateTimes('chatMessageDataLog', jsonData.lastUpdateTimes) ) {
                this.analyzeChatMessageDataLog(jsonData.chatMessageDataLog);
            }
            
            if( jsonData.characters &&
                sender.checkLastUpdateTimes('characters', jsonData.lastUpdateTimes) ) {
                Log.logging("jsonData.characters", jsonData.characters);
                Log.logging("jsonData.characters.length", jsonData.characters.length);
                
                this.analyzeCharacterData(jsonData.characters);
            }
            
            if( jsonData.roundTimeData &&
                sender.checkLastUpdateTimes('time', jsonData.lastUpdateTimes) ) {
                this.analyzeTime(jsonData.roundTimeData);
            }
            
            if( jsonData.mapData &&
                sender.checkLastUpdateTimes('map', jsonData.lastUpdateTimes) ) {
                this.analyzeMap(jsonData.mapData);
            }
            
            if( jsonData.effects &&
                sender.checkLastUpdateTimes('effects', jsonData.lastUpdateTimes) ) {
                this.analyzeEffects(jsonData.effects);
            }
            
            if( isFirstRefresh() ) {
                initForFirstRefresh();
            }
            
            return true;
        }
        
        private function analyzeEffects(effects:Object):void {
            var tmp:ArrayCollection = new ArrayCollection(effects as Array);
            DodontoF_Main.getInstance().setEffects( tmp );
        }
        
        private function analyzeChatMessageDataLog(chatMessageDataLog:Object):void {
            Log.logging("chatMessageDataLog", chatMessageDataLog);
            
            var lastWrittenTime:Number = this.chatMessageDataLastWrittenTime;
            
            for(var i:int = 0 ; i < chatMessageDataLog.length ; i++) {
                var chatMessageDataSet:Object = chatMessageDataLog[i];
                
                var writtenTime:Number = chatMessageDataSet[0];
                var chatMessageData:Object = chatMessageDataSet[1];
                
                if( this.chatMessageDataLastWrittenTime >= writtenTime ) {
                    Log.logging("this chatLog message is NOT new.");
                    continue;
                }
                
                Log.logging("this is new chatLog message.");
                
                if( writtenTime > lastWrittenTime ) {
                    lastWrittenTime = writtenTime;
                }
                
                var senderName:String = chatMessageData['senderName'];
                var chatMessage:String = chatMessageData['message'];
                var color:String = chatMessageData['color'];
                var chatSenderUniqueId:String = chatMessageData['uniqueId'];
                var messageIndex:int = chatMessageData['messageIndex'];
                var sendto:String = chatMessageData['sendto'];
                
                DodontoF_Main.getInstance().getChatWindow()
                    .addMessageToChatLogParts(senderName, chatMessage,
                                              color, writtenTime, chatSenderUniqueId,
                                              messageIndex, sendto);
                
                Log.logging("chatLog oneLine end");
            }
            
            DodontoF_Main.getInstance().getChatWindow().printAddedMessageToChatMessageLog();
            
            this.chatMessageDataLastWrittenTime = lastWrittenTime;
        }
        
        private function analyzeMap(mapData:Object):void {
            if( mapData.mapType != "imageGraphic" ) {
                return;
            }
            
            map.changeMap(mapData.imageSource, mapData.xMax, mapData.yMax, mapData.gridColor);
            map.changeMarks(mapData.mapMarks);
        }
        
        
        private function analyzeCharacterData(characterDataList:Object):void {
            Log.loggingTuning("=>Begin analyzeCharacterData");
            Log.logging("analyzeCharacterData characterDataList", characterDataList);
            
            Log.loggingTuning("=>Begin analyzeRemoveCharacter");
            this.analyzeRemoveCharacter(characterDataList);
            Log.loggingTuning("=>End analyzeRemoveCharacter");
            
            for(var i:int = 0 ; i < characterDataList.length ; i++) {
                var characterData:Object = characterDataList[i];
                Log.logging("characterData is ", characterData);
                this.analyzeAddCharacter(characterData);
                Log.loggingTuning("=>analyzeAddCharacter");
                this.analyzeMoveCharacter(characterData);
                Log.loggingTuning("=>analyzeMoveCharacter");
                this.analyzeChangedCharacter(characterData);
                Log.loggingTuning("=>analyzeChangedCharacter");
            }
            Log.loggingTuning("=>End analyzeCharacterData pre");
            
            this.refreshInitiativeList();
            ChatWindow.getInstance().refreshChatCharacterName();
            
            Log.loggingTuning("=>End analyzeCharacterData");
        }
        
        private function analyzeAddCharacter(characterData:Object):void {
            Log.logging("analyzeAddCharacter( characterData)", characterData);
        
            if( map.findExistCharacter(characterData) != null ) {
                Log.logging("character already exist, so pass charactr create");
                return;
            }
            
            Log.logging("existCharacter check end...");
            addCharacterInOwnMap(characterData);
            
            Log.logging("this.existPieces.push( character )");
        }
        
        public function addCharacterInOwnMap(characterData:Object):void {
            Log.logging("SharedDataReceiver.addCharacterInOwnMap() begin");
            var piece:Piece = this.createPieceFromCharacterData(characterData);
            Log.logging("piece", piece);
            
            if( piece == null ) {
                Log.loggingError("invalid Character", JSON.encode(characterData));
                if( characterData.imgId != null ) {
                    sender.removeInvalidCharacter(characterData.imgId);
                    Log.loggingError("不正なデータが含まれていたため自動削除しました。");
                }
                
                return;
            }
            
            Log.logging("characterData", characterData);
            Log.logging("movablePiece addCharacter");
            map.addExistPieces( piece );
            pickupAddedForReplayMode(piece);
            
            Log.logging("SharedDataReceiver.addCharacterInOwnMap() end");
        }
        
        private function pickupAddedForReplayMode(piece:Piece):void {
            if( ! DodontoF_Main.getInstance().isReplayMode() ) {
                return;
            }
            
            var pickupTarget:InitiativedMovablePiece = piece as InitiativedMovablePiece;
            if( pickupTarget == null ) {
                return;
            }
            
            pickupTarget.pickup();
        }
            
        
        public function removeCharacterOnlyOwnMap(event:Event):void {
            var existList:Array = [];
            
            for(var i:int = 0 ; i < map.getExistPiecesCount() ; i++) {
                var piece:Piece = map.getExistPiece(i);
                if( (piece != null ) && piece.isOnlyOwnMap() ) {
                    piece.remove();
                } else {
                    existList.push( piece );
                }
            }
            
            map.setExistPieces(existList);
        }
        
        public function startHistory():void {
            history = new Array();
            isHistoryOn = true;
        }
        
        public function stopHistory():Boolean {
            if( ! isHistoryOn ) {
                return false;
            }
            
            isHistoryOn = false;
            return true;
        }
        
        public function getHistory():Array {
            return history;
        }
        
        private function createPieceFromCharacterData(characterData:Object):Piece {
            var piece:Piece = null;
            
            if( characterData.type == MagicRange.getTypeStatic() ) {
                piece = new MagicRange(characterData);
            } else if( characterData.type == MagicRangeDD4th.getTypeStatic() ) {
                piece = new MagicRangeDD4th(characterData);
            } else if( characterData.type == MapMask.getTypeStatic() ) {
                piece = new MapMask(characterData);
            } else if( characterData.type == MapMarker.getTypeStatic() ) {
                piece = new MapMarker(characterData);
            } else if( characterData.type == DiceSymbol.getTypeStatic() ) {
                piece = new DiceSymbol(characterData);
            } else if( characterData.type == MagicTimer.getTypeStatic() ) {
                piece = new MagicTimer(characterData);
            } else if( characterData.type == Card.getTypeStatic() ) {
                piece = new Card(characterData);
            } else if( characterData.type == CardMount.getTypeStatic() ) {
                piece = new CardMount(characterData);
            } else if( characterData.type == CardTrushMount.getTypeStatic() ) {
                piece = new CardTrushMount(characterData);
            } else if( characterData.type == Character.getTypeStatic() ) {
                piece = new Character(characterData);
                /*
            } else if( characterData.type == StatusMarker.getTypeStatic() ) {
                piece = new StatusMarker(characterData);
                */
            } else {
                var message:String = "SharedDataReceiver.createCharacterFromCharacterData() Unknown characterData receivered.";
                Log.loggingError( message + "imgId", characterData.imgId);
                Log.loggingError( message + "type", characterData.type);
                return null;
            }
            
            piece.init(map, characterData.x, characterData.y);
            
            return piece;
        }
        
        
        private function analyzeRemoveCharacter(characterDataList:Object):void {
            var existList:Array = [];
            for(var i:int = 0 ; i < map.getExistPiecesCount() ; i++) {
                var existCharacter:Piece = map.getExistPiece(i);
                Log.logging("analyzeRemoveCharacter existCharacter ", existCharacter);
                
                var exist:Object = findExistCharacterData(characterDataList, existCharacter);
                
                if( exist == null ) {
                    Log.logging("character is NOT exist, so remove", existCharacter);
                    existCharacter.remove();
                } else {
                    Log.logging("existed character", existCharacter);
                    existList.push( existCharacter );
                }
            }
            
            map.setExistPieces(existList);
        }
        
        private function findExistCharacterData(characterDataList:Object, existCharacter:Piece):Object {
            Log.logging("findExistCharacterData start");
            
            for(var i:int = 0 ; i < characterDataList.length ; i++) {
                Log.logging("characterData gettting....");
                var characterData:Object = characterDataList[i];
                Log.logging("characterData get.");
                
                if( characterData.imgId == existCharacter.getId() ) {
                    return characterData;
                }
            }
            
            return null;
        }
        
        private function analyzeMoveCharacter(characterData:Object):void {
            Log.logging("analyzeMoveCharacter characterData", characterData);
            var character:Piece = map.findExistCharacter(characterData);
            Log.logging("moveCharacter character", character);
            
            if( character == null ) {
                Log.logging("target is NOT character.");
                return;
            }
            Log.logging("target is character.");
            
            var isMoved:Boolean = character.move(characterData.x, characterData.y);
            pickupMovedForReplayMode(isMoved, character);
            
            Log.logging("character moved");
        }
        
        private function pickupMovedForReplayMode(isMoved:Boolean, piece:Piece):void {
            if( ! isMoved ) {
                return;
            }
            
            if( ! DodontoF_Main.getInstance().isReplayMode() ) {
                return;
            }
            
            var pickupTarget:InitiativedMovablePiece = piece as InitiativedMovablePiece;
            if( pickupTarget == null ) {
                return;
            }
            
            pickupTarget.pickup();
        }
        
        private function analyzeChangedCharacter(characterData:Object):void {
            Log.logging("analyzeChangedCharacter start");
            
            var character:Piece = map.findExistCharacter(characterData);
            if( character != null ) {
                Log.logging("SharedDataSender.analyzeChangedCharacter(characterData)", characterData);
                character.analyzeChangedCharacter(characterData);
            }
            Log.logging("analyzeChangedCharacter end");
        }


        private function analyzeTime(roundTimeData:Object):void {
            this.roundTimer.setExistCharacters( map.getExistPieces() );
            this.roundTimer.setCounterNames(roundTimeData.counterNames);
            this.roundTimer.setTime(roundTimeData.round,
                                    roundTimeData.initiative);
            this.refreshInitiativeList();
        }
        
        
        private function refreshInitiativeList():void {
            Log.logging("refreshInitiativeList() begin");
            roundTimer.setExistCharacters( map.getExistPieces() );
            roundTimer.refreshInitiativeList();
            Log.logging("refreshInitiativeList() end");
        }
        

        /*
        public function saveResult(event:Event):void {
            Log.loggingTuning("saveResult begin");
            try {
                var jsonData:Object = getJsonDataFromResultEvent(event);
                Log.loggingError("jsonData", jsonData);
                var saveData:Object = jsonData.saveData;
                Log.loggingError("saveData", saveData);
                var saveDataString:String = JSON.encode(jsonData);
                Log.loggingError("saveDataString", saveDataString);
                
                var fileReferenceForSave:FileReference = new FileReference();
                fileReferenceForSave.addEventListener(Event.COMPLETE, function(event:Event):void {});
                fileReferenceForSave.save(saveDataString, "DodontoF.sav");
            } catch(e:Error) {
                Log.loggingException("SharedDataReceiver.saveResult()", e);
            }
            Log.loggingTuning("saveResult end");
        }
        */

    }
}

