//--*-coding:utf-8-*--

package {
    
    import mx.controls.Alert;
    import flash.display.Loader;
    import flash.display.LoaderInfo;
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.MouseEvent;
    import flash.net.URLRequest;
    import mx.controls.Image;
    import mx.core.UIComponent;
    
    
    public class StandingGraphics {
        
        private var standingGraphicInfosOriginal:Array = new Array();
        private var images:Array = new Array(getMaxLeftIndex());
        private var baseX:int = 0;
        private var baseY:int = 0;
        private var baseWidth:int = 0;
        private var displayState:Boolean = true;
        private var noTargetImageAplha:Number = 0.7;
        
        
        static public function getMaxLeftIndex():int {
            return 12;
        }
        
        public function clear():void {
            standingGraphicInfosOriginal = new Array();
        }
        
        public function add(info:Object):void {
            var name:String = info.name;
            var state:String = info.state;
            var source:String = info.source;
            var leftIndex:int = info.leftIndex;
            if( leftIndex == 0 ) {
                leftIndex = 1;
            }
            
            if( findTargetSource(name, state) != null ) {
                Log.loggingError("キャラクター名：" + name
                                 + "、状態：" + state 
                                 + "、はすでに登録済みの立ち絵が存在します。");
                return;
            }
            
            var info:Object = {
                "name" : name,
                "state" : state,
                "source" : source,
                "leftIndex" : leftIndex
            };
            
            standingGraphicInfosOriginal.push(info);
        }
        
        public function isDisplayStateOn():Boolean {
            return displayState;
        }
        
        public function setDisplayState(b:Boolean):void {
            displayState = b;
            
            if( isDisplayStateOn() ) {
                return;
            }
            
            clearExistImages();
        }
        
        
        private function pushStandingGraphicInfosFromCharacters(infos:Array):void {
            var map:Map = DodontoF_Main.getInstance().getMap();
            var characters:Array = map.getCharacters();
            
            for(var i:int = 0 ; i < characters.length ; i++) {
                var character:Character = characters[i];
                
                var characterInfo:Object = {
                    "name" : character.getName(),
                    "state" : "",
                    "source" : character.getImageUrl()
                };
                infos.push(characterInfo);
            }
        }
        
        private function getStandingGraphicInfos(pushCharacterInfoFinction:Function = null):Array {
            var infos:Array = new Array();
            
            if( ! isDisplayStateOn() ) {
                return infos;
            }
            
            for(var i:int = 0 ; i < standingGraphicInfosOriginal.length ; i++) {
                var info:Object = standingGraphicInfosOriginal[i];
                infos.push(info);
            }
            
            if( pushCharacterInfoFinction == null ) {
                pushCharacterInfoFinction = pushStandingGraphicInfosFromCharacters;
            }
            
            pushCharacterInfoFinction(infos);
            
            return infos;
        }
        
        public function findAllStates(name:String):Array {
            var stateList:Array = new Array();
            
            var standingGraphicInfos:Array = getStandingGraphicInfos();
            for(var i:int = 0 ; i < standingGraphicInfos.length ; i++) {
                var info:Object = standingGraphicInfos[i];
                if( name == info.name ) {
                    if( info.state != "" ) {
                        stateList.push(info.state);
                    }
                }
            }
            
            return stateList;
        }
        
        public function findAllNames():Array {
            var names:Array = new Array();
            
            var standingGraphicInfos:Array = getStandingGraphicInfos();
            for(var i:int = 0 ; i < standingGraphicInfos.length ; i++) {
                var info:Object = standingGraphicInfos[i];
                if( names.indexOf(info.name) == -1 ) {
                    names.push(info.name);
                }
            }
                
            return names;
        }
        
        public function findTargetSource(name:String,
                                         state:String,
                                         chatMessage:String = "",
                                         pushCharacterInfoFinction:Function = null):String {
            var info:Object = findTargetInfo(name, state, chatMessage, pushCharacterInfoFinction);
            if( info == null ) {
                return null;
            }
            
            return info.source;
        }
        
        public function findTargetInfo(name:String,
                                       state:String,
                                       chatMessage:String = "",
                                       pushCharacterInfoFinction:Function = null):Object {
            var result:Object = null;
            Log.logging("chatMessage", chatMessage);
            
            var standingGraphicInfos:Array = getStandingGraphicInfos( pushCharacterInfoFinction );
            for(var i:int = 0 ; i < standingGraphicInfos.length ; i++) {
                var info:Object = standingGraphicInfos[i];
                if( name == info.name ) {
                    if( state == info.state ) {
                        result = info;
                    }
                    
                    if( chatMessage == "" ) {
                        continue;
                    }
                    
                    if( info.state == "" ) {
                        continue;
                    }
                    
                    if( isHitTailAnyLine(chatMessage, info.state) ) {
                        result = info;
                        break;
                    }
                }
            }
            
            return result;
        }
        
        static private var chatLineSpliter:RegExp = /(\r|\n)/;
        
        private function isHitTailAnyLine(linesString:String, tail:String):Boolean {
            var lines:Array = linesString.split( chatLineSpliter );
            
            for(var i:int = 0 ; i < lines.length ; i++) {
                var line:String = lines[i];
                
                var index:int = line.search(tail);
                if( index == -1 ) {
                    continue;
                }
                
                if( index == (line.length - tail.length) ) {
                    return true;
                }
            }
            
            return false;
        }
        
        public function shiftAllImagesYPosition(diff:int):void {
            for(var i:int = 0 ; i < this.images.length ; i++) {
                var component:UIComponent = this.images[i] as UIComponent;
                if( component == null ) {
                    continue;
                }
                
                component.y = diff;
            }
        }
        
        public function clearExistImages(leftIndex:int = -1):void {
            for(var i:int = 0 ; i < this.images.length ; i++) {
                
                var component:UIComponent = this.images[i] as UIComponent;
                if( component == null ) {
                    continue;
                }
                
                if( (leftIndex == i) || (leftIndex == -1) ) {
                    removeImageComponent(component, leftIndex);
                    this.images[leftIndex] = null;
                    continue;
                }
                
                component.alpha = noTargetImageAplha;
            }
        }
        
        private function removeImageComponent(component:UIComponent, leftIndex:int):void {
            try {
                var dodontoFComp:UIComponent = DodontoF_Main.getInstance().getStandingGraphicLayer();
                dodontoFComp.removeChild( component );
            } catch(e:Error) {
                //Log.loggingException("StandingGraphicInfos:clear", e);
            }
        }
        
        static public function getTypeStatic():String {
            return "standingGraphicInfos";
        }
        
        public function print(nameBase:String,
                              chatMessage:String,
                              effectable:Boolean,
                              filterImages:Array,
                              chatWindowX:int,
                              chatWindowY:int,
                              chatWindowWidth:int):Object {
            var name:String = nameBase;
            var state:String = "";
            
            var params:Array = nameBase.split(/\t/);
            if(params.length == 2) {
                name = params[0];
                state = params[1];
            }
            
            var result:Object = {
                "senderName" : name
            }
            
            if( name == "" ) {
                return result;
            }
            
            var info:Object = findTargetInfo(name, state, chatMessage);
            if(info == null) {
                //clearExistImages();
                return result;
            }
            
            var leftIndex:int = info.leftIndex;
            if( leftIndex == 0 ) {
                leftIndex = 1;
            }
            
            if( ! effectable ) {
                clearExistImages(leftIndex);
                return result;
            }
            
            var source:String = info.source;
            source = Config.getInstance().getUrlString(source);
            
            //ウィンドサイズの分高さを調整。
            baseY = chatWindowY - 30;

            //カットインが左端だと見栄えが悪いので調整。
            baseX = chatWindowX + 10;
            
            //幅一杯だと見栄えが悪いので調整
            baseWidth = chatWindowWidth - 10;
            
            var imageCompleteHandlerFunction:Function = getImageCompleteHandler(leftIndex, filterImages);
            var existEvent:Event = imageLoaderEventHash[source] as Event;
            if( existEvent != null ) {
                imageCompleteHandlerFunction(existEvent);
            } else {
                loadImage(source, imageCompleteHandlerFunction);
            }
            
            return result;
        }
        
        private function loadImage(source:String, imageCompleteHandlerFunction:Function):void {
            source = Config.getInstance().getUrlString(source);
            
            var imageLoader:Loader = new Loader();
            imageLoader.contentLoaderInfo.addEventListener(Event.COMPLETE, imageCompleteHandlerFunction);
            imageLoader.load(new URLRequest(source));
        }
        
        private function getImageCompleteHandler(leftIndex:int, filterImages:Array):Function {
            return function(event:Event):void {
                imageCompleteHandler(event, leftIndex, filterImages);
            };
        }
        
        private var imageLoaderEventHash:Object = new Object();
        
        private function saveEventHash(event:Event):void {
            var loaderInfo:LoaderInfo = event.currentTarget as LoaderInfo;
            var source:String = loaderInfo.url;
            imageLoaderEventHash[source] = event;
        }
        
        private var imageWidthRate:Number = 0.4;
        
        private function imageCompleteHandler(event:Event, leftIndex:int, filterImages:Array):void {
            saveEventHash(event);
            
            var imageLoader:Loader = event.target.loader;
            
            var imageHeigthMinimum:int = 200;
            var imageInfo:Object = CutInMovie.getSizeInfo(imageLoader, 0, 0, imageHeigthMinimum);
            
            var image:Loader = imageLoader;
            image.height = imageInfo.height;
            image.width = imageInfo.width;
            
            if( Config.getInstance().isAdjustImageSizeMode() ) {
                if( image.height < imageHeigthMinimum ) {
                    setImageSizeFromHeigth(imageHeigthMinimum, image);
                }
                
                var imageHeigthMax:int = baseY - 50;//baseY * 0.9;
                if( image.height > imageHeigthMax ) {
                    setImageSizeFromHeigth(imageHeigthMax, image);
                }
                
                var imageWidthMax:int = baseWidth * imageWidthRate;
                if( image.width > imageWidthMax ) {
                    setImageSizeFromWidth(imageWidthMax, image);
                }
            }
            
            
            var leftIndexRate:Number = ((leftIndex - 1) / 12);
            var leftPadding:Number = ((baseWidth - image.width) * leftIndexRate);
            image.x = baseX + leftPadding;
            image.y = baseY - image.height;
            
            addImage(leftIndex, imageLoader, filterImages);
        }
        
        private function addImage(leftIndex:int, imageLoader:Loader, filterImages:Array):void {
            var component:UIComponent = new UIComponent();
            component.addChild(imageLoader);
            
            var dodontoFComp:UIComponent = DodontoF_Main.getInstance().getStandingGraphicLayer();
            dodontoFComp.addChild(component);
            clearExistImages(leftIndex);
            this.images[leftIndex] = component;
            
            component.addEventListener(MouseEvent.CLICK, function(event:MouseEvent):void {
                    dodontoFComp.removeChild(component);
                });
            
            for(var i:int = 0 ; i < filterImages.length ; i++) {
                loadImage(filterImages[i], getImageCompFundlerForFilter(imageLoader, component));
            }
        }
        
        private function getImageCompFundlerForFilter(baseLoader:Loader, component:UIComponent):Function {
            return function(event:Event):void {
                var loader:Loader = event.target.loader;
                loader.x = baseLoader.x;
                loader.y = baseLoader.y;
                //フィルター画像は正方形にしたいので、あえてwidthにheightを代入する。
                loader.width = baseLoader.height;
                loader.height = baseLoader.height;
                
                component.addChild(loader);
            }
        }
        
        
        private function setImageSizeFromHeigth(height:int,
                                                image:Loader):void {
            var rate:Number = (height / image.height);
            image.width = image.width * rate;
            image.height = height;
        }
        private function setImageSizeFromWidth(width:int,
                                               image:Loader):void {
            var rate:Number = (width / image.width);
            image.width = width;
            image.height = image.height * rate;
        }
    }
}
